@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2023 NVIDIA Corporation
+    Copyright (c) 2015-2024 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -86,11 +86,13 @@ static void init_tools_data(uvm_va_space_t *va_space)
 
     for (i = 0; i < ARRAY_SIZE(va_space->tools.counters); i++)
         INIT_LIST_HEAD(va_space->tools.counters + i);
-    for (i = 0; i < ARRAY_SIZE(va_space->tools.queues); i++)
-        INIT_LIST_HEAD(va_space->tools.queues + i);
+    for (i = 0; i < ARRAY_SIZE(va_space->tools.queues_v1); i++)
+        INIT_LIST_HEAD(va_space->tools.queues_v1 + i);
+    for (i = 0; i < ARRAY_SIZE(va_space->tools.queues_v2); i++)
+        INIT_LIST_HEAD(va_space->tools.queues_v2 + i);
 }
 
-static NV_STATUS register_gpu_nvlink_peers(uvm_va_space_t *va_space, uvm_gpu_t *gpu)
+static NV_STATUS register_gpu_peers(uvm_va_space_t *va_space, uvm_gpu_t *gpu)
 {
     uvm_gpu_t *other_gpu;
 
@@ -104,7 +106,7 @@ static NV_STATUS register_gpu_nvlink_peers(uvm_va_space_t *va_space, uvm_gpu_t *
 
         peer_caps = uvm_gpu_peer_caps(gpu, other_gpu);
 
-        if (peer_caps->link_type >= UVM_GPU_LINK_NVLINK_1) {
+        if (peer_caps->link_type >= UVM_GPU_LINK_NVLINK_1 || gpu->parent == other_gpu->parent) {
             NV_STATUS status = enable_peers(va_space, gpu, other_gpu);
             if (status != NV_OK)
                 return status;
@@ -117,15 +119,27 @@ static NV_STATUS register_gpu_nvlink_peers(uvm_va_space_t *va_space, uvm_gpu_t *
 static bool va_space_check_processors_masks(uvm_va_space_t *va_space)
 {
     uvm_processor_id_t processor;
-    uvm_processor_mask_t processors;
+    uvm_va_block_context_t *block_context = uvm_va_space_block_context(va_space, NULL);
+    uvm_processor_mask_t *processors = &block_context->scratch_processor_mask;
 
     uvm_assert_rwsem_locked_write(&va_space->lock);
 
-    uvm_processor_mask_copy(&processors, &va_space->registered_gpus);
-    uvm_processor_mask_set(&processors, UVM_ID_CPU);
+    uvm_processor_mask_copy(processors, &va_space->registered_gpus);
+    uvm_processor_mask_set(processors, UVM_ID_CPU);
 
-    for_each_id_in_mask(processor, &processors) {
+    for_each_id_in_mask(processor, processors) {
         uvm_processor_id_t other_processor;
+        bool check_can_copy_from = true;
+
+        if (UVM_ID_IS_GPU(processor)) {
+            uvm_gpu_t *gpu = uvm_va_space_get_gpu(va_space, processor);
+
+            // Peer copies between two processors can be disabled even when they
+            // are NvLink peers, or there is HW support for atomics between
+            // them.
+            if (gpu->parent->peer_copy_mode == UVM_GPU_PEER_COPY_MODE_UNSUPPORTED)
+                check_can_copy_from = false;
+        }
 
         UVM_ASSERT(processor_mask_array_test(va_space->can_access, processor, processor));
         UVM_ASSERT(processor_mask_array_test(va_space->accessible_from, processor, processor));
@@ -135,8 +149,11 @@ static bool va_space_check_processors_masks(uvm_va_space_t *va_space)
 
         // NVLINK
         UVM_ASSERT(!processor_mask_array_test(va_space->has_nvlink, processor, processor));
-        UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_nvlink[uvm_id_value(processor)],
-                                             &va_space->can_copy_from[uvm_id_value(processor)]));
+
+        if (check_can_copy_from) {
+            UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_nvlink[uvm_id_value(processor)],
+                                                 &va_space->can_copy_from[uvm_id_value(processor)]));
+        }
 
         // Peers
         UVM_ASSERT(!processor_mask_array_test(va_space->indirect_peers, processor, processor));
@@ -145,8 +162,12 @@ static bool va_space_check_processors_masks(uvm_va_space_t *va_space)
 
         // Atomics
         UVM_ASSERT(processor_mask_array_test(va_space->has_native_atomics, processor, processor));
-        UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_native_atomics[uvm_id_value(processor)],
-                                             &va_space->can_copy_from[uvm_id_value(processor)]));
+
+        if (check_can_copy_from) {
+            UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_native_atomics[uvm_id_value(processor)],
+                                                 &va_space->can_copy_from[uvm_id_value(processor)]));
+        }
+
         UVM_ASSERT(uvm_processor_mask_subset(&va_space->has_native_atomics[uvm_id_value(processor)],
                                              &va_space->can_access[uvm_id_value(processor)]));
 
@@ -176,6 +197,7 @@ NV_STATUS uvm_va_space_create(struct address_space *mapping, uvm_va_space_t **va
     }
 
     uvm_init_rwsem(&va_space->lock, UVM_LOCK_ORDER_VA_SPACE);
+    uvm_mutex_init(&va_space->closest_processors.mask_mutex, UVM_LOCK_ORDER_LEAF);
     uvm_mutex_init(&va_space->serialize_writers_lock, UVM_LOCK_ORDER_VA_SPACE_SERIALIZE_WRITERS);
     uvm_mutex_init(&va_space->read_acquire_write_release_lock,
                    UVM_LOCK_ORDER_VA_SPACE_READ_ACQUIRE_WRITE_RELEASE_LOCK);
@@ -283,7 +305,7 @@ static void unregister_gpu(uvm_va_space_t *va_space,
                            uvm_gpu_t *gpu,
                            struct mm_struct *mm,
                            struct list_head *deferred_free_list,
-                           uvm_global_processor_mask_t *peers_to_release)
+                           uvm_processor_mask_t *peers_to_release)
 {
     uvm_gpu_t *peer_gpu;
     uvm_va_range_t *va_range;
@@ -292,7 +314,7 @@ static void unregister_gpu(uvm_va_space_t *va_space,
     uvm_assert_rwsem_locked_write(&va_space->lock);
 
     if (peers_to_release)
-        uvm_global_processor_mask_zero(peers_to_release);
+        uvm_processor_mask_zero(peers_to_release);
 
     // If a GPU VA Space was explicitly registered, but not explicitly
     // unregistered, unregister it and add all of its objects to the free list.
@@ -320,14 +342,19 @@ static void unregister_gpu(uvm_va_space_t *va_space,
             // brought up and torn down automatically within add_gpu and
             // remove_gpu.
             if (peers_to_release && g_uvm_global.peers[peer_table_index].link_type == UVM_GPU_LINK_PCIE)
-                uvm_global_processor_mask_set(peers_to_release, peer_gpu->global_id);
+                uvm_processor_mask_set(peers_to_release, peer_gpu->id);
         }
     }
 
-    if (gpu->parent->isr.replayable_faults.handling)
+    if (gpu->parent->isr.replayable_faults.handling) {
+        UVM_ASSERT(uvm_processor_mask_test(&va_space->faultable_processors, gpu->id));
         uvm_processor_mask_clear(&va_space->faultable_processors, gpu->id);
-
-    uvm_processor_mask_clear(&va_space->system_wide_atomics_enabled_processors, gpu->id);
+        uvm_processor_mask_clear(&va_space->system_wide_atomics_enabled_processors, gpu->id);
+    }
+    else {
+        UVM_ASSERT(uvm_processor_mask_test(&va_space->non_faultable_processors, gpu->id));
+        uvm_processor_mask_clear(&va_space->non_faultable_processors, gpu->id);
+    }
 
     processor_mask_array_clear(va_space->can_access, gpu->id, gpu->id);
     processor_mask_array_clear(va_space->can_access, gpu->id, UVM_ID_CPU);
@@ -419,8 +446,7 @@ void uvm_va_space_destroy(uvm_va_space_t *va_space)
     uvm_va_range_t *va_range, *va_range_next;
     uvm_gpu_t *gpu;
     uvm_gpu_id_t gpu_id;
-    uvm_global_gpu_id_t global_gpu_id;
-    uvm_global_processor_mask_t retained_gpus;
+    uvm_processor_mask_t *retained_gpus = &va_space->registered_gpus_teardown;
     LIST_HEAD(deferred_free_list);
 
     // Remove the VA space from the global list before we start tearing things
@@ -448,7 +474,7 @@ void uvm_va_space_destroy(uvm_va_space_t *va_space)
     // registered GPUs in the VA space, so those faults will be canceled.
     uvm_va_space_down_write(va_space);
 
-    uvm_va_space_global_gpus(va_space, &retained_gpus);
+    uvm_processor_mask_copy(retained_gpus, &va_space->registered_gpus);
 
     bitmap_copy(va_space->enabled_peers_teardown, va_space->enabled_peers, UVM_MAX_UNIQUE_GPU_PAIRS);
 
@@ -500,7 +526,7 @@ void uvm_va_space_destroy(uvm_va_space_t *va_space)
 
     nv_kthread_q_flush(&g_uvm_global.global_q);
 
-    for_each_global_gpu_in_mask(gpu, &retained_gpus) {
+    for_each_gpu_in_mask(gpu, retained_gpus) {
         if (!gpu->parent->isr.replayable_faults.handling) {
             UVM_ASSERT(!gpu->parent->isr.non_replayable_faults.handling);
             continue;
@@ -515,7 +541,16 @@ void uvm_va_space_destroy(uvm_va_space_t *va_space)
             nv_kthread_q_flush(&gpu->parent->isr.kill_channel_q);
 
         if (gpu->parent->access_counters_supported)
-            uvm_gpu_access_counters_disable(gpu, va_space);
+            uvm_parent_gpu_access_counters_disable(gpu->parent, va_space);
+
+        // Free the processor masks allocated in uvm_va_space_register_gpu().
+        // The mask is also freed in uvm_va_space_unregister_gpu() but that
+        // function won't be called in uvm_release() and uvm_release_deferred()
+        // path.
+        uvm_processor_mask_cache_free(va_space->peers_to_release[uvm_id_value(gpu->id)]);
+
+        // Set the pointer to NULL to avoid accidental re-use and double free.
+        va_space->peers_to_release[uvm_id_value(gpu->id)] = NULL;
     }
 
     // Check that all CPU/GPU affinity masks are empty
@@ -545,16 +580,16 @@ void uvm_va_space_destroy(uvm_va_space_t *va_space)
     uvm_mutex_lock(&g_uvm_global.global_lock);
 
     // Release the GPUs and their peer counts. Do not use
-    // for_each_global_gpu_in_mask for the outer loop as it reads the GPU
+    // for_each_gpu_in_mask for the outer loop as it reads the GPU
     // state, which might get destroyed.
-    for_each_global_gpu_id_in_mask(global_gpu_id, &retained_gpus) {
+    for_each_gpu_id_in_mask(gpu_id, retained_gpus) {
         uvm_gpu_t *peer_gpu;
 
-        gpu = uvm_gpu_get(global_gpu_id);
+        gpu = uvm_gpu_get(gpu_id);
 
-        uvm_global_processor_mask_clear(&retained_gpus, global_gpu_id);
+        uvm_processor_mask_clear(retained_gpus, gpu_id);
 
-        for_each_global_gpu_in_mask(peer_gpu, &retained_gpus) {
+        for_each_gpu_in_mask(peer_gpu, retained_gpus) {
             NvU32 peer_table_index = uvm_gpu_peer_table_index(gpu->id, peer_gpu->id);
             if (test_bit(peer_table_index, va_space->enabled_peers_teardown)) {
                 uvm_gpu_peer_t *peer_caps = &g_uvm_global.peers[peer_table_index];
@@ -605,7 +640,7 @@ uvm_gpu_t *uvm_va_space_get_gpu_by_uuid(uvm_va_space_t *va_space, const NvProces
     uvm_gpu_t *gpu;
 
     for_each_va_space_gpu(gpu, va_space) {
-        if (uvm_processor_uuid_eq(uvm_gpu_uuid(gpu), gpu_uuid))
+        if (uvm_uuid_eq(&gpu->uuid, gpu_uuid))
             return gpu;
     }
 
@@ -641,47 +676,52 @@ uvm_gpu_t *uvm_va_space_retain_gpu_by_uuid(uvm_va_space_t *va_space, const NvPro
 
 bool uvm_va_space_can_read_duplicate(uvm_va_space_t *va_space, uvm_gpu_t *changing_gpu)
 {
-    uvm_processor_mask_t changing_gpu_mask;
-    uvm_processor_mask_t non_faultable_gpus;
-    uvm_processor_mask_t registered_gpu_va_spaces;
+    NvU32 count = va_space->num_non_faultable_gpu_va_spaces;
 
-    uvm_processor_mask_zero(&changing_gpu_mask);
+    if (changing_gpu && !uvm_processor_mask_test(&va_space->faultable_processors, changing_gpu->id)) {
+        if (uvm_processor_mask_test(&va_space->registered_gpu_va_spaces, changing_gpu->id)) {
+            // A non-faultable GPU is getting removed.
+            UVM_ASSERT(count > 0);
+            --count;
+        }
+        else {
+            // A non-faultable GPU is getting added.
+            ++count;
+        }
+    }
 
-    if (changing_gpu)
-        uvm_processor_mask_set(&changing_gpu_mask, changing_gpu->id);
-
-    // flip the bit of the changing GPU to represent the state change in progress
-    uvm_processor_mask_xor(&registered_gpu_va_spaces, &changing_gpu_mask, &va_space->registered_gpu_va_spaces);
-
-    // Can't enable read-duplication if any non-fault-capable GPUs have GPU VA spaces registered
-    return !uvm_processor_mask_andnot(&non_faultable_gpus, &registered_gpu_va_spaces, &va_space->faultable_processors);
+    return count == 0;
 }
 
-// Note that the "VA space" in the function name refers to a UVM per-process VA space.
-// (This is different from a per-GPU VA space.)
+// Note that the "VA space" in the function name refers to a UVM per-process
+// VA space. (This is different from a per-GPU VA space.)
 NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
                                     const NvProcessorUuid *gpu_uuid,
                                     const uvm_rm_user_object_t *user_rm_device,
                                     NvBool *numa_enabled,
-                                    NvS32 *numa_node_id)
+                                    NvS32 *numa_node_id,
+                                    NvProcessorUuid *uuid_out)
 {
     NV_STATUS status;
     uvm_va_range_t *va_range;
     uvm_gpu_t *gpu;
     uvm_gpu_t *other_gpu;
     bool gpu_can_access_sysmem = true;
+    uvm_processor_mask_t *peers_to_release = NULL;
 
     status = uvm_gpu_retain_by_uuid(gpu_uuid, user_rm_device, &gpu);
     if (status != NV_OK)
         return status;
 
+    uvm_uuid_copy(uuid_out, &gpu->uuid);
+
     // Enabling access counters requires taking the ISR lock, so it is done
     // without holding the (deeper order) VA space lock. Enabling the counters
     // after dropping the VA space lock would create a window of time in which
     // another thread could see the GPU as registered, but access counters would
     // be disabled. Therefore, the counters are enabled before taking the VA
     // space lock.
-    if (uvm_gpu_access_counters_required(gpu->parent)) {
+    if (uvm_parent_gpu_access_counters_required(gpu->parent)) {
         status = uvm_gpu_access_counters_enable(gpu, va_space);
         if (status != NV_OK) {
             uvm_gpu_release(gpu);
@@ -711,7 +751,7 @@ NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
         goto done;
     }
 
-    if (uvm_conf_computing_mode_enabled(gpu)) {
+    if (g_uvm_global.conf_computing_enabled) {
         NvU32 gpu_index = uvm_id_gpu_index(gpu->id);
         status = uvm_conf_computing_dma_buffer_alloc(&gpu->conf_computing.dma_buffer_pool,
                                                      &va_space->gpu_unregister_dma_buffer[gpu_index],
@@ -722,14 +762,31 @@ NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
         gpu_can_access_sysmem = false;
     }
 
+    UVM_ASSERT(!va_space->peers_to_release[uvm_id_value(gpu->id)]);
+
+    peers_to_release = uvm_processor_mask_cache_alloc();
+    if (!peers_to_release) {
+        status = NV_ERR_NO_MEMORY;
+        goto done;
+    }
+
+    va_space->peers_to_release[uvm_id_value(gpu->id)] = peers_to_release;
+
     uvm_processor_mask_set(&va_space->registered_gpus, gpu->id);
     va_space->registered_gpus_table[uvm_id_gpu_index(gpu->id)] = gpu;
 
     if (gpu->parent->isr.replayable_faults.handling) {
+        UVM_ASSERT(!uvm_processor_mask_test(&va_space->faultable_processors, gpu->id));
         uvm_processor_mask_set(&va_space->faultable_processors, gpu->id);
+
+        UVM_ASSERT(!uvm_processor_mask_test(&va_space->system_wide_atomics_enabled_processors, gpu->id));
         // System-wide atomics are enabled by default
         uvm_processor_mask_set(&va_space->system_wide_atomics_enabled_processors, gpu->id);
     }
+    else {
+        UVM_ASSERT(!uvm_processor_mask_test(&va_space->non_faultable_processors, gpu->id));
+        uvm_processor_mask_set(&va_space->non_faultable_processors, gpu->id);
+    }
 
     // All GPUs have native atomics on their own memory
     processor_mask_array_set(va_space->has_native_atomics, gpu->id, gpu->id);
@@ -785,13 +842,11 @@ NV_STATUS uvm_va_space_register_gpu(uvm_va_space_t *va_space,
         }
     }
 
-    status = register_gpu_nvlink_peers(va_space, gpu);
+    status = register_gpu_peers(va_space, gpu);
     if (status != NV_OK)
         goto cleanup;
 
-    status = uvm_perf_heuristics_register_gpu(va_space, gpu);
-    if (status != NV_OK)
-        goto cleanup;
+    uvm_perf_heuristics_register_gpu(va_space, gpu);
 
     uvm_for_each_va_range(va_range, va_space) {
         status = uvm_va_range_register_gpu(va_range, gpu);
@@ -816,6 +871,10 @@ cleanup:
     // a deferred_free_list, mm, etc.
     unregister_gpu(va_space, gpu, NULL, NULL, NULL);
 
+    va_space->peers_to_release[uvm_id_value(gpu->id)] = NULL;
+
+    uvm_processor_mask_cache_free(peers_to_release);
+
 done:
     UVM_ASSERT(va_space_check_processors_masks(va_space));
 
@@ -824,9 +883,9 @@ done:
     if (status != NV_OK) {
         // There is no risk of disabling access counters on a previously
         // registered GPU: the enablement step would have failed before even
-        // discovering that the GPU is already registed.
-        if (uvm_gpu_access_counters_required(gpu->parent))
-            uvm_gpu_access_counters_disable(gpu, va_space);
+        // discovering that the GPU is already registered.
+        if (uvm_parent_gpu_access_counters_required(gpu->parent))
+            uvm_parent_gpu_access_counters_disable(gpu->parent, va_space);
 
         uvm_gpu_release(gpu);
     }
@@ -839,8 +898,8 @@ NV_STATUS uvm_va_space_unregister_gpu(uvm_va_space_t *va_space, const NvProcesso
     uvm_gpu_t *gpu;
     uvm_gpu_va_space_t *gpu_va_space;
     struct mm_struct *mm;
-    uvm_global_gpu_id_t peer_gpu_id;
-    uvm_global_processor_mask_t peers_to_release;
+    uvm_gpu_id_t peer_gpu_id;
+    uvm_processor_mask_t *peers_to_release;
     LIST_HEAD(deferred_free_list);
 
     // Stopping channels requires holding the VA space lock in read mode, so do
@@ -878,15 +937,16 @@ NV_STATUS uvm_va_space_unregister_gpu(uvm_va_space_t *va_space, const NvProcesso
     // it from the VA space until we're done.
     uvm_va_space_up_read_rm(va_space);
 
-    // If uvm_gpu_access_counters_required(gpu->parent) is true, a concurrent
-    // registration could enable access counters after they are disabled here.
+    // If uvm_parent_gpu_access_counters_required(gpu->parent) is true, a
+    // concurrent registration could enable access counters after they are
+    // disabled here.
     // The concurrent registration will fail later on if it acquires the VA
     // space lock before the unregistration does (because the GPU is still
     // registered) and undo the access counters enablement, or succeed if it
     // acquires the VA space lock after the unregistration does. Both outcomes
     // result on valid states.
     if (gpu->parent->access_counters_supported)
-        uvm_gpu_access_counters_disable(gpu, va_space);
+        uvm_parent_gpu_access_counters_disable(gpu->parent, va_space);
 
     // mmap_lock is needed to establish CPU mappings to any pages evicted from
     // the GPU if accessed by CPU is set for them.
@@ -900,8 +960,12 @@ NV_STATUS uvm_va_space_unregister_gpu(uvm_va_space_t *va_space, const NvProcesso
     if (uvm_processor_mask_test(&va_space->registered_gpu_va_spaces, gpu->id))
         UVM_ASSERT(uvm_gpu_va_space_get(va_space, gpu) == gpu_va_space);
 
+    peers_to_release = va_space->peers_to_release[uvm_id_value(gpu->id)];
+
+    va_space->peers_to_release[uvm_id_value(gpu->id)] = NULL;
+
     // This will call disable_peers for all GPU's peers, including NVLink
-    unregister_gpu(va_space, gpu, mm, &deferred_free_list, &peers_to_release);
+    unregister_gpu(va_space, gpu, mm, &deferred_free_list, peers_to_release);
 
     UVM_ASSERT(uvm_processor_mask_test(&va_space->gpu_unregister_in_progress, gpu->id));
     uvm_processor_mask_clear(&va_space->gpu_unregister_in_progress, gpu->id);
@@ -920,14 +984,18 @@ NV_STATUS uvm_va_space_unregister_gpu(uvm_va_space_t *va_space, const NvProcesso
     // Release the VA space's GPU and peer counts
     uvm_mutex_lock(&g_uvm_global.global_lock);
 
-    // Do not use for_each_global_gpu_in_mask as it reads the peer GPU state,
+    // Do not use for_each_gpu_in_mask as it reads the peer GPU state,
     // which might get destroyed when we release the peer entry.
-    for_each_global_gpu_id_in_mask(peer_gpu_id, &peers_to_release) {
+    UVM_ASSERT(peers_to_release);
+
+    for_each_gpu_id_in_mask(peer_gpu_id, peers_to_release) {
         uvm_gpu_t *peer_gpu = uvm_gpu_get(peer_gpu_id);
         UVM_ASSERT(uvm_gpu_peer_caps(gpu, peer_gpu)->link_type == UVM_GPU_LINK_PCIE);
         uvm_gpu_release_pcie_peer_access(gpu, peer_gpu);
     }
 
+    uvm_processor_mask_cache_free(peers_to_release);
+
     uvm_gpu_release_locked(gpu);
 
     uvm_mutex_unlock(&g_uvm_global.global_lock);
@@ -1009,7 +1077,6 @@ static NV_STATUS enable_peers(uvm_va_space_t *va_space, uvm_gpu_t *gpu0, uvm_gpu
         return NV_ERR_NOT_COMPATIBLE;
     }
 
-    // TODO: Bug 3848497: Disable GPU Peer Mapping when HCC is enabled
     processor_mask_array_set(va_space->can_access, gpu0->id, gpu1->id);
     processor_mask_array_set(va_space->can_access, gpu1->id, gpu0->id);
     processor_mask_array_set(va_space->accessible_from, gpu0->id, gpu1->id);
@@ -1042,6 +1109,10 @@ static NV_STATUS enable_peers(uvm_va_space_t *va_space, uvm_gpu_t *gpu0, uvm_gpu
             processor_mask_array_set(va_space->indirect_peers, gpu1->id, gpu0->id);
         }
     }
+    else if (gpu0->parent == gpu1->parent) {
+        processor_mask_array_set(va_space->has_native_atomics, gpu0->id, gpu1->id);
+        processor_mask_array_set(va_space->has_native_atomics, gpu1->id, gpu0->id);
+    }
 
     UVM_ASSERT(va_space_check_processors_masks(va_space));
     __set_bit(table_index, va_space->enabled_peers);
@@ -1080,7 +1151,7 @@ static NV_STATUS retain_pcie_peers_from_uuids(uvm_va_space_t *va_space,
     *gpu0 = uvm_va_space_get_gpu_by_uuid(va_space, gpu_uuid_1);
     *gpu1 = uvm_va_space_get_gpu_by_uuid(va_space, gpu_uuid_2);
 
-    if (*gpu0 && *gpu1 && !uvm_id_equal((*gpu0)->id, (*gpu1)->id))
+    if (*gpu0 && *gpu1 && !uvm_parent_id_equal((*gpu0)->parent->id, (*gpu1)->parent->id))
         status = uvm_gpu_retain_pcie_peer_access(*gpu0, *gpu1);
     else
         status = NV_ERR_INVALID_DEVICE;
@@ -1093,6 +1164,7 @@ static NV_STATUS retain_pcie_peers_from_uuids(uvm_va_space_t *va_space,
 static bool uvm_va_space_pcie_peer_enabled(uvm_va_space_t *va_space, uvm_gpu_t *gpu0, uvm_gpu_t *gpu1)
 {
     return !processor_mask_array_test(va_space->has_nvlink, gpu0->id, gpu1->id) &&
+           gpu0->parent != gpu1->parent &&
            uvm_va_space_peer_enabled(va_space, gpu0, gpu1);
 }
 
@@ -1336,7 +1408,7 @@ static NV_STATUS create_gpu_va_space(uvm_gpu_t *gpu,
                                 gpu_va_space,
                                 UVM_PAGE_TREE_TYPE_USER,
                                 gpu_address_space_info.bigPageSize,
-                                uvm_gpu_page_tree_init_location(gpu),
+                                uvm_get_page_tree_location(gpu->parent),
                                 &gpu_va_space->page_tables);
     if (status != NV_OK) {
         UVM_ERR_PRINT("Initializing the page tree failed: %s, GPU %s\n", nvstatusToString(status), uvm_gpu_name(gpu));
@@ -1363,6 +1435,9 @@ static void add_gpu_va_space(uvm_gpu_va_space_t *gpu_va_space)
     UVM_ASSERT(va_space);
     uvm_assert_rwsem_locked_write(&va_space->lock);
 
+    if (!uvm_processor_mask_test(&va_space->faultable_processors, gpu->id))
+        va_space->num_non_faultable_gpu_va_spaces++;
+
     uvm_processor_mask_set(&va_space->registered_gpu_va_spaces, gpu->id);
     va_space->gpu_va_spaces[uvm_id_gpu_index(gpu->id)] = gpu_va_space;
     gpu_va_space->state = UVM_GPU_VA_SPACE_STATE_ACTIVE;
@@ -1511,6 +1586,49 @@ error_gpu_release:
     return status;
 }
 
+static NvU32 find_gpu_va_space_index(uvm_va_space_t *va_space,
+                                     uvm_parent_gpu_t *parent_gpu)
+{
+    uvm_gpu_id_t gpu_id;
+    NvU32 index = UVM_ID_MAX_PROCESSORS;
+
+    // TODO: Bug 4351121: this conversion from parent ID to gpu ID depends on
+    // the fact that only one partition is registered per va_space per physical
+    // GPU. This code will need to change when multiple MIG instances are
+    // supported.
+    for_each_sub_processor_id_in_parent_gpu(gpu_id, parent_gpu->id) {
+        if (uvm_processor_mask_test(&va_space->registered_gpu_va_spaces, gpu_id)) {
+            UVM_ASSERT(index == UVM_ID_MAX_PROCESSORS);
+            index = uvm_id_gpu_index(gpu_id);
+        }
+    }
+
+    return index;
+}
+
+uvm_gpu_va_space_t *uvm_gpu_va_space_get_by_parent_gpu(uvm_va_space_t *va_space,
+                                                       uvm_parent_gpu_t *parent_gpu)
+{
+    uvm_gpu_va_space_t *gpu_va_space;
+    NvU32 gpu_index;
+
+    uvm_assert_rwsem_locked(&va_space->lock);
+
+    if (!parent_gpu)
+        return NULL;
+
+    gpu_index = find_gpu_va_space_index(va_space, parent_gpu);
+    if (gpu_index == UVM_ID_MAX_PROCESSORS)
+        return NULL;
+
+    gpu_va_space = va_space->gpu_va_spaces[gpu_index];
+    UVM_ASSERT(uvm_gpu_va_space_state(gpu_va_space) == UVM_GPU_VA_SPACE_STATE_ACTIVE);
+    UVM_ASSERT(gpu_va_space->va_space == va_space);
+    UVM_ASSERT(gpu_va_space->gpu->parent == parent_gpu);
+
+    return gpu_va_space;
+}
+
 // The caller must have stopped all channels under this gpu_va_space before
 // calling this function.
 static void remove_gpu_va_space(uvm_gpu_va_space_t *gpu_va_space,
@@ -1520,6 +1638,7 @@ static void remove_gpu_va_space(uvm_gpu_va_space_t *gpu_va_space,
     uvm_va_space_t *va_space;
     uvm_va_range_t *va_range;
     uvm_va_range_t *va_range_next;
+    uvm_gpu_t *gpu;
 
     if (!gpu_va_space || uvm_gpu_va_space_state(gpu_va_space) != UVM_GPU_VA_SPACE_STATE_ACTIVE)
         return;
@@ -1550,8 +1669,15 @@ static void remove_gpu_va_space(uvm_gpu_va_space_t *gpu_va_space,
     // space to be destroyed.
     atomic_inc(&va_space->gpu_va_space_deferred_free.num_pending);
 
-    uvm_processor_mask_clear(&va_space->registered_gpu_va_spaces, gpu_va_space->gpu->id);
-    va_space->gpu_va_spaces[uvm_id_gpu_index(gpu_va_space->gpu->id)] = NULL;
+    gpu = gpu_va_space->gpu;
+
+    if (!uvm_processor_mask_test(&va_space->faultable_processors, gpu->id)) {
+        UVM_ASSERT(va_space->num_non_faultable_gpu_va_spaces);
+        va_space->num_non_faultable_gpu_va_spaces--;
+    }
+
+    uvm_processor_mask_clear(&va_space->registered_gpu_va_spaces, gpu->id);
+    va_space->gpu_va_spaces[uvm_id_gpu_index(gpu->id)] = NULL;
     gpu_va_space->state = UVM_GPU_VA_SPACE_STATE_DEAD;
 }
 
@@ -1635,49 +1761,59 @@ uvm_processor_id_t uvm_processor_mask_find_closest_id(uvm_va_space_t *va_space,
                                                       const uvm_processor_mask_t *candidates,
                                                       uvm_processor_id_t src)
 {
-    uvm_processor_mask_t mask;
-    uvm_processor_id_t id;
+    uvm_processor_mask_t *mask = &va_space->closest_processors.mask;
+    uvm_processor_id_t closest_id;
 
     // Highest priority: the local processor itself
     if (uvm_processor_mask_test(candidates, src))
         return src;
 
-    // NvLink peers
-    if (uvm_processor_mask_and(&mask, candidates, &va_space->has_nvlink[uvm_id_value(src)])) {
+    uvm_mutex_lock(&va_space->closest_processors.mask_mutex);
+
+    if (uvm_processor_mask_and(mask, candidates, &va_space->has_nvlink[uvm_id_value(src)])) {
+        // NvLink peers
         uvm_processor_mask_t *indirect_peers;
-        uvm_processor_mask_t direct_peers;
+        uvm_processor_mask_t *direct_peers = &va_space->closest_processors.direct_peers;
 
         indirect_peers = &va_space->indirect_peers[uvm_id_value(src)];
 
-        // Direct peers, prioritizing GPU peers over CPU
-        if (uvm_processor_mask_andnot(&direct_peers, &mask, indirect_peers)) {
-            id = uvm_processor_mask_find_first_gpu_id(&direct_peers);
-            return UVM_ID_IS_INVALID(id)? UVM_ID_CPU : id;
+        if (uvm_processor_mask_andnot(direct_peers, mask, indirect_peers)) {
+            // Direct peers, prioritizing GPU peers over CPU
+            closest_id = uvm_processor_mask_find_first_gpu_id(direct_peers);
+            if (UVM_ID_IS_INVALID(closest_id))
+                closest_id = UVM_ID_CPU;
         }
+        else {
+            // Indirect peers
+            UVM_ASSERT(UVM_ID_IS_GPU(src));
+            UVM_ASSERT(!uvm_processor_mask_test(mask, UVM_ID_CPU));
 
-        // Indirect peers
-        UVM_ASSERT(UVM_ID_IS_GPU(src));
-        UVM_ASSERT(!uvm_processor_mask_test(&mask, UVM_ID_CPU));
-
-        return uvm_processor_mask_find_first_gpu_id(&mask);
+            closest_id = uvm_processor_mask_find_first_gpu_id(mask);
+        }
     }
-
-    // If source is GPU, prioritize PCIe peers over CPU
-    if (uvm_processor_mask_and(&mask, candidates, &va_space->can_access[uvm_id_value(src)])) {
+    else if (uvm_processor_mask_and(mask, candidates, &va_space->can_access[uvm_id_value(src)])) {
+        // If source is GPU, prioritize PCIe peers over CPU
         // CPUs only have direct access to GPU memory over NVLINK, not PCIe, and
         // should have been selected above
         UVM_ASSERT(UVM_ID_IS_GPU(src));
 
-        id = uvm_processor_mask_find_first_gpu_id(&mask);
-        return UVM_ID_IS_INVALID(id)? UVM_ID_CPU : id;
+        closest_id = uvm_processor_mask_find_first_gpu_id(mask);
+        if (UVM_ID_IS_INVALID(closest_id))
+            closest_id = UVM_ID_CPU;
+    }
+    else {
+        // No GPUs with direct access are in the mask. Just pick the first
+        // processor in the mask, if any.
+        closest_id = uvm_processor_mask_find_first_id(candidates);
     }
 
-    // No GPUs with direct access are in the mask. Just pick the first
-    // processor in the mask, if any.
-    return uvm_processor_mask_find_first_id(candidates);
+    uvm_mutex_unlock(&va_space->closest_processors.mask_mutex);
+
+    return closest_id;
 }
 
-static void uvm_deferred_free_object_channel(uvm_deferred_free_object_t *object, uvm_processor_mask_t *flushed_gpus)
+static void uvm_deferred_free_object_channel(uvm_deferred_free_object_t *object,
+                                             uvm_parent_processor_mask_t *flushed_parent_gpus)
 {
     uvm_user_channel_t *channel = container_of(object, uvm_user_channel_t, deferred_free);
     uvm_gpu_t *gpu = channel->gpu;
@@ -1685,9 +1821,10 @@ static void uvm_deferred_free_object_channel(uvm_deferred_free_object_t *object,
     // Flush out any faults with this instance pointer still in the buffer. This
     // prevents us from re-allocating the same instance pointer for a new
     // channel and mis-attributing old faults to it.
-    if (gpu->parent->replayable_faults_supported && !uvm_processor_mask_test(flushed_gpus, gpu->id)) {
+    if (gpu->parent->replayable_faults_supported &&
+        !uvm_parent_processor_mask_test(flushed_parent_gpus, gpu->parent->id)) {
         uvm_gpu_fault_buffer_flush(gpu);
-        uvm_processor_mask_set(flushed_gpus, gpu->id);
+        uvm_parent_processor_mask_set(flushed_parent_gpus, gpu->parent->id);
     }
 
     uvm_user_channel_destroy_detached(channel);
@@ -1696,17 +1833,20 @@ static void uvm_deferred_free_object_channel(uvm_deferred_free_object_t *object,
 void uvm_deferred_free_object_list(struct list_head *deferred_free_list)
 {
     uvm_deferred_free_object_t *object, *next;
-    uvm_processor_mask_t flushed_gpus;
+    uvm_parent_processor_mask_t flushed_parent_gpus;
 
-    // Used if there are any channels in the list
-    uvm_processor_mask_zero(&flushed_gpus);
+    // flushed_parent_gpus prevents redundant fault buffer flushes by tracking
+    // the parent GPUs on which the flush already happened. Flushing the fault
+    // buffer on one GPU instance will flush it for all other instances on that
+    // parent GPU.
+    uvm_parent_processor_mask_zero(&flushed_parent_gpus);
 
     list_for_each_entry_safe(object, next, deferred_free_list, list_node) {
         list_del(&object->list_node);
 
         switch (object->type) {
             case UVM_DEFERRED_FREE_OBJECT_TYPE_CHANNEL:
-                uvm_deferred_free_object_channel(object, &flushed_gpus);
+                uvm_deferred_free_object_channel(object, &flushed_parent_gpus);
                 break;
             case UVM_DEFERRED_FREE_OBJECT_GPU_VA_SPACE:
                 destroy_gpu_va_space(container_of(object, uvm_gpu_va_space_t, deferred_free));
@@ -2069,11 +2209,55 @@ NV_STATUS uvm_test_destroy_gpu_va_space_delay(UVM_TEST_DESTROY_GPU_VA_SPACE_DELA
     return NV_OK;
 }
 
+NV_STATUS uvm_test_force_cpu_to_cpu_copy_with_ce(UVM_TEST_FORCE_CPU_TO_CPU_COPY_WITH_CE_PARAMS *params,
+                                                 struct file *filp)
+
+{
+    uvm_va_space_t *va_space = uvm_va_space_get(filp);
+
+    va_space->test.force_cpu_to_cpu_copy_with_ce = params->force_copy_with_ce;
+    return NV_OK;
+}
+
+NV_STATUS uvm_test_va_space_allow_movable_allocations(UVM_TEST_VA_SPACE_ALLOW_MOVABLE_ALLOCATIONS_PARAMS *params,
+                                                      struct file *filp)
+{
+    uvm_va_space_t *va_space = uvm_va_space_get(filp);
+
+    va_space->test.allow_allocation_from_movable = params->allow_movable;
+    return NV_OK;
+}
+
 // List of fault service contexts for CPU faults
 static LIST_HEAD(g_cpu_service_block_context_list);
 
 static uvm_spinlock_t g_cpu_service_block_context_list_lock;
 
+uvm_service_block_context_t *uvm_service_block_context_alloc(struct mm_struct *mm)
+{
+    uvm_service_block_context_t *service_context = uvm_kvmalloc(sizeof(*service_context));
+
+    if (!service_context)
+        return NULL;
+
+    service_context->block_context = uvm_va_block_context_alloc(mm);
+    if (!service_context->block_context) {
+        uvm_kvfree(service_context);
+        service_context = NULL;
+    }
+
+    return service_context;
+}
+
+void uvm_service_block_context_free(uvm_service_block_context_t *service_context)
+{
+    if (!service_context)
+        return;
+
+    uvm_va_block_context_free(service_context->block_context);
+    uvm_kvfree(service_context);
+}
+
 NV_STATUS uvm_service_block_context_init(void)
 {
     unsigned num_preallocated_contexts = 4;
@@ -2082,17 +2266,11 @@ NV_STATUS uvm_service_block_context_init(void)
 
     // Pre-allocate some fault service contexts for the CPU and add them to the global list
     while (num_preallocated_contexts-- > 0) {
-        uvm_service_block_context_t *service_context = uvm_kvmalloc(sizeof(*service_context));
+        uvm_service_block_context_t *service_context = uvm_service_block_context_alloc(NULL);
 
         if (!service_context)
             return NV_ERR_NO_MEMORY;
 
-        service_context->block_context = uvm_va_block_context_alloc(NULL);
-        if (!service_context->block_context) {
-            uvm_kvfree(service_context);
-            return NV_ERR_NO_MEMORY;
-        }
-
         list_add(&service_context->cpu_fault.service_context_list, &g_cpu_service_block_context_list);
     }
 
@@ -2104,11 +2282,13 @@ void uvm_service_block_context_exit(void)
     uvm_service_block_context_t *service_context, *service_context_tmp;
 
     // Free fault service contexts for the CPU and add clear the global list
-    list_for_each_entry_safe(service_context, service_context_tmp, &g_cpu_service_block_context_list,
+    list_for_each_entry_safe(service_context,
+                             service_context_tmp,
+                             &g_cpu_service_block_context_list,
                              cpu_fault.service_context_list) {
-        uvm_va_block_context_free(service_context->block_context);
-        uvm_kvfree(service_context);
+        uvm_service_block_context_free(service_context);
     }
+
     INIT_LIST_HEAD(&g_cpu_service_block_context_list);
 }
 
@@ -2120,7 +2300,8 @@ static uvm_service_block_context_t *service_block_context_cpu_alloc(void)
 
     uvm_spin_lock(&g_cpu_service_block_context_list_lock);
 
-    service_context = list_first_entry_or_null(&g_cpu_service_block_context_list, uvm_service_block_context_t,
+    service_context = list_first_entry_or_null(&g_cpu_service_block_context_list,
+                                               uvm_service_block_context_t,
                                                cpu_fault.service_context_list);
 
     if (service_context)
@@ -2128,17 +2309,10 @@ static uvm_service_block_context_t *service_block_context_cpu_alloc(void)
 
     uvm_spin_unlock(&g_cpu_service_block_context_list_lock);
 
-    if (!service_context) {
-        service_context = uvm_kvmalloc(sizeof(*service_context));
-        service_context->block_context = uvm_va_block_context_alloc(NULL);
-        if (!service_context->block_context) {
-            uvm_kvfree(service_context);
-            service_context = NULL;
-        }
-    }
-    else {
+    if (!service_context)
+        service_context = uvm_service_block_context_alloc(NULL);
+    else
         uvm_va_block_context_init(service_context->block_context, NULL);
-    }
 
     return service_context;
 }
@@ -2166,7 +2340,7 @@ static vm_fault_t uvm_va_space_cpu_fault(uvm_va_space_t *va_space,
     bool major_fault = false;
     bool is_remote_mm = false;
     uvm_service_block_context_t *service_context;
-    uvm_global_processor_mask_t gpus_to_check_for_ecc;
+    uvm_processor_mask_t *gpus_to_check_for_ecc;
 
     if (status != NV_OK)
         goto convert_error;
@@ -2299,20 +2473,17 @@ static vm_fault_t uvm_va_space_cpu_fault(uvm_va_space_t *va_space,
     }
 
     tools_enabled = va_space->tools.enabled;
+    gpus_to_check_for_ecc = &service_context->cpu_fault.gpus_to_check_for_ecc;
 
-    if (status == NV_OK && !is_remote_mm) {
-        uvm_va_space_global_gpus_in_mask(va_space,
-                                         &gpus_to_check_for_ecc,
-                                         &service_context->cpu_fault.gpus_to_check_for_ecc);
-        uvm_global_mask_retain(&gpus_to_check_for_ecc);
-    }
+    if (status == NV_OK && !is_remote_mm)
+        uvm_global_gpu_retain(gpus_to_check_for_ecc);
 
     uvm_va_space_up_read(va_space);
     uvm_record_unlock_mmap_lock_read(vma->vm_mm);
 
     if (status == NV_OK && !is_remote_mm) {
-        status = uvm_global_mask_check_ecc_error(&gpus_to_check_for_ecc);
-        uvm_global_mask_release(&gpus_to_check_for_ecc);
+        status = uvm_global_gpu_check_ecc_error(gpus_to_check_for_ecc);
+        uvm_global_gpu_release(gpus_to_check_for_ecc);
     }
 
     if (tools_enabled)
