@@ -63,7 +63,7 @@
 #define NV_DP_REGKEY_POWER_DOWN_PHY                   "DP_POWER_DOWN_PHY"
 
 //
-// Regkey to re-assess max link if the first assessed link config 
+// Regkey to re-assess max link if the first assessed link config
 // is lower than the panel max
 //
 #define NV_DP_REGKEY_REASSESS_MAX_LINK                "DP_REASSESS_MAX_LINK"
@@ -74,6 +74,16 @@
 //
 #define NV_DP_DSC_MST_CAP_BUG_3143315                  "DP_DSC_MST_CAP_BUG_3143315"
 
+//
+// Bug 4388987 : This regkey will disable reading PCON caps for MST.
+//
+#define NV_DP_REGKEY_MST_PCON_CAPS_READ_DISABLED       "DP_BUG_4388987_WAR"
+
+//
+// Bug 4459839 : This regkey will enable DSC irrespective of LT status.
+//
+#define NV_DP_REGKEY_FORCE_DSC_ON_SINK                 "DP_FORCE_DSC_ON_SINK"
+
 //
 // Data Base used to store all the regkey values.
 // The actual data base is declared statically in dp_evoadapter.cpp.
@@ -107,6 +117,8 @@ struct DP_REGKEY_DATABASE
     bool  bDscMstCapBug3143315;
     bool  bPowerDownPhyBeforeD3;
     bool  bReassessMaxLink;
+    bool  bMSTPCONCapsReadDisabled;
+    bool  bForceDscOnSink;
 };
 
 #endif //INCLUDED_DP_REGKEYDATABASE_H
