@@ -35,8 +35,6 @@
 #include "dp_vrr.h"
 #include <nvmisc.h>
 
-#include <ctrl/ctrl0073/ctrl0073dfp.h>
-#include <ctrl/ctrl0073/ctrl0073dp.h>
 #include <ctrl/ctrl0073/ctrl0073specific.h>
 #include <ctrl/ctrl0073/ctrl0073system.h>
 #include <ctrl/ctrl5070/ctrl5070or.h>
@@ -95,7 +93,9 @@ const struct
     {NV_DP_REGKEY_FORCE_EDP_ILR,                    &dpRegkeyDatabase.bBypassEDPRevCheck,              DP_REG_VAL_BOOL},
     {NV_DP_DSC_MST_CAP_BUG_3143315,                 &dpRegkeyDatabase.bDscMstCapBug3143315,            DP_REG_VAL_BOOL},
     {NV_DP_REGKEY_POWER_DOWN_PHY,                   &dpRegkeyDatabase.bPowerDownPhyBeforeD3,           DP_REG_VAL_BOOL},
-    {NV_DP_REGKEY_REASSESS_MAX_LINK,                &dpRegkeyDatabase.bReassessMaxLink,                DP_REG_VAL_BOOL}
+    {NV_DP_REGKEY_REASSESS_MAX_LINK,                &dpRegkeyDatabase.bReassessMaxLink,                DP_REG_VAL_BOOL},
+    {NV_DP_REGKEY_MST_PCON_CAPS_READ_DISABLED,      &dpRegkeyDatabase.bMSTPCONCapsReadDisabled,        DP_REG_VAL_BOOL},
+    {NV_DP_REGKEY_FORCE_DSC_ON_SINK,                &dpRegkeyDatabase.bForceDscOnSink,                 DP_REG_VAL_BOOL},
 };
 
 EvoMainLink::EvoMainLink(EvoInterface * provider, Timer * timer) :
@@ -113,10 +113,12 @@ EvoMainLink::EvoMainLink(EvoInterface * provider, Timer * timer) :
     this->initializeRegkeyDatabase();
     this->applyRegkeyOverrides();
 
-     _isDynamicMuxCapable       = false;
-     _isLTPhyRepeaterSupported  = true;
-     _rmPhyRepeaterCount        = 0;
-     dpMemZero(&_DSC, sizeof(_DSC));
+    _isDynamicMuxCapable       = false;
+    _isLTPhyRepeaterSupported  = true;
+    _rmPhyRepeaterCount        = 0;
+    dpMemZero(&_DSC, sizeof(_DSC));
+    dpMemZero(&dfpParams, sizeof(dfpParams));
+    dpMemZero(&dpParams, sizeof(dpParams));
 
     //
     //  Tell RM to hands off on the DisplayPort hardware
@@ -257,63 +259,60 @@ NvU32 EvoMainLink::headToStream(NvU32 head, DP_SINGLE_HEAD_MULTI_STREAM_PIPELINE
     return streamIndex;
 }
 
-void EvoMainLink::queryGPUCapability()
+bool EvoMainLink::queryGPUCapability()
 {
-    NV0073_CTRL_CMD_DP_GET_CAPS_PARAMS params;
-
-    dpMemZero(&params, sizeof(params));
-    params.subDeviceInstance = subdeviceIndex;
-    params.sorIndex = provider->getSorIndex();
-    NvU32 code = provider->rmControl0073(NV0073_CTRL_CMD_DP_GET_CAPS, &params, sizeof(params));
+    dpMemZero(&dpParams, sizeof(dpParams));
+    dpParams.subDeviceInstance = subdeviceIndex;
+    dpParams.sorIndex = provider->getSorIndex();
+    NvU32 code = provider->rmControl0073(NV0073_CTRL_CMD_DP_GET_CAPS, &dpParams, sizeof(dpParams));
     if (code != NVOS_STATUS_SUCCESS)
     {
         DP_ASSERT(0 && "Unable to process GPU caps");
+        return false;
     }
+    //
+    // Check if MST feature needs to be disabled by regkey. This is requirement by few OEMs, they don't want to support
+    // MST feature on particular sku, whenever requested through INF.
+    //
+    _hasMultistream         = (dpParams.bIsMultistreamSupported == NV_TRUE) && !_isMstDisabledByRegkey;
+
+    _gpuSupportedDpVersions = dpParams.dpVersionsSupported;
+
+    _isStreamCloningEnabled = (dpParams.bIsSCEnabled == NV_TRUE) ? true : false;
+    _hasIncreasedWatermarkLimits     = (dpParams.bHasIncreasedWatermarkLimits == NV_TRUE) ? true : false;
+
+    _isFECSupported         = (dpParams.bFECSupported == NV_TRUE) ? true : false;
+
+    _useDfpMaxLinkRateCaps  = (dpParams.bOverrideLinkBw == NV_TRUE) ? true : false;
+
+    _isLTPhyRepeaterSupported        = (dpParams.bIsTrainPhyRepeater == NV_TRUE) ? true : false;
+
+    if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _1_62, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = RBR; //in Hz
+    else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _2_70, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = HBR; //in Hz
+    else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _5_40, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = HBR2; //in Hz
+    else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _8_10, dpParams.maxLinkRate))
+        _maxLinkRateSupportedGpu = HBR3; //in Hz
     else
     {
-        //
-        // Check if MST feature needs to be disabled by regkey. This is requirement by few OEMs, they don't want to support
-        // MST feature on particular sku, whenever requested through INF.
-        //
-        _hasMultistream         = (params.bIsMultistreamSupported == NV_TRUE) && !_isMstDisabledByRegkey;
-
-        _gpuSupportedDpVersions = params.dpVersionsSupported;
-
-        _isStreamCloningEnabled = (params.bIsSCEnabled == NV_TRUE) ? true : false;
-        _hasIncreasedWatermarkLimits     = (params.bHasIncreasedWatermarkLimits == NV_TRUE) ? true : false;
-
-        _isFECSupported         = (params.bFECSupported == NV_TRUE) ? true : false;
-
-        _useDfpMaxLinkRateCaps  = (params.bOverrideLinkBw == NV_TRUE) ? true : false;
-
-        _isLTPhyRepeaterSupported        = (params.bIsTrainPhyRepeater == NV_TRUE) ? true : false;
-
-        if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _1_62, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = RBR; //in Hz
-        else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _2_70, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = HBR; //in Hz
-        else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _5_40, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = HBR2; //in Hz
-        else if (FLD_TEST_DRF(0073, _CTRL_CMD_DP_GET_CAPS, _MAX_LINK_RATE, _8_10, params.maxLinkRate))
-            _maxLinkRateSupportedGpu = HBR3; //in Hz
-        else
-        {
-            DP_ASSERT(0 && "Unable to get max link rate");
-            // Assume that we can at least support RBR.
-            _maxLinkRateSupportedGpu = RBR;
-        }
-
-        if (!_isDscDisabledByRegkey)
-        {
-            _DSC.isDscSupported = params.DSC.bDscSupported ? true : false;
-            _DSC.encoderColorFormatMask = params.DSC.encoderColorFormatMask;
-            _DSC.lineBufferSizeKB = params.DSC.lineBufferSizeKB;
-            _DSC.rateBufferSizeKB = params.DSC.rateBufferSizeKB;
-            _DSC.bitsPerPixelPrecision = params.DSC.bitsPerPixelPrecision;
-            _DSC.maxNumHztSlices = params.DSC.maxNumHztSlices;
-            _DSC.lineBufferBitDepth = params.DSC.lineBufferBitDepth;
-        }
+        DP_ASSERT(0 && "Unable to get max link rate");
+        // Assume that we can at least support RBR.
+        _maxLinkRateSupportedGpu = RBR;
     }
+
+    if (!_isDscDisabledByRegkey)
+    {
+        _DSC.isDscSupported = dpParams.DSC.bDscSupported ? true : false;
+        _DSC.encoderColorFormatMask = dpParams.DSC.encoderColorFormatMask;
+        _DSC.lineBufferSizeKB = dpParams.DSC.lineBufferSizeKB;
+        _DSC.rateBufferSizeKB = dpParams.DSC.rateBufferSizeKB;
+        _DSC.bitsPerPixelPrecision = dpParams.DSC.bitsPerPixelPrecision;
+        _DSC.maxNumHztSlices = dpParams.DSC.maxNumHztSlices;
+        _DSC.lineBufferBitDepth = dpParams.DSC.lineBufferBitDepth;
+    }
+    return true;
 }
 
 void EvoMainLink::triggerACT()
@@ -643,18 +642,18 @@ bool EvoMainLink::physicalLayerSetTestPattern(PatternInfo * patternInfo)
         {
             ctrlPattern.testPattern = NV0073_CTRL_DP_TESTPATTERN_DATA_CSTM;
 
-            params.cstm.lower = patternInfo->ctsmLower;
-            params.cstm.middle = patternInfo->ctsmMiddle;
-            params.cstm.upper = patternInfo->ctsmUpper;
+            params.cstm.field_31_0 = patternInfo->ctsmLower;
+            params.cstm.field_63_32 = patternInfo->ctsmMiddle;
+            params.cstm.field_95_64 = patternInfo->ctsmUpper;
             break;
         }
 #ifdef NV0073_CTRL_DP_TESTPATTERN_DATA_HBR2COMPLIANCE
         case LINK_QUAL_HBR2_COMPLIANCE_EYE:
         {
             ctrlPattern.testPattern = NV0073_CTRL_DP_TESTPATTERN_DATA_HBR2COMPLIANCE;
-            params.cstm.lower = 0;
-            params.cstm.middle = 0;
-            params.cstm.upper = 0;
+            params.cstm.field_31_0 = 0;
+            params.cstm.field_63_32 = 0;
+            params.cstm.field_95_64 = 0;
             break;
         }
 #endif
@@ -893,6 +892,7 @@ void EvoMainLink::applyRegkeyOverrides()
     _skipPowerdownEDPPanelWhenHeadDetach = dpRegkeyDatabase.bPoweroffEdpInHeadDetachSkipped;
     _applyLinkBwOverrideWarRegVal        = dpRegkeyDatabase.bLinkBwOverrideWarApplied;
     _enableMSAOverrideOverMST            = dpRegkeyDatabase.bMsaOverMstEnabled;
+    _isMSTPCONCapsReadDisabled           = dpRegkeyDatabase.bMSTPCONCapsReadDisabled;
 }
 
 NvU32 EvoMainLink::getRegkeyValue(const char *key)
@@ -1373,7 +1373,6 @@ bool EvoMainLink::getMaxLinkConfigFromUefi(NvU8 &linkRate, NvU8 &laneCount)
 
 bool EvoMainLink::queryAndUpdateDfpParams()
 {
-    NV0073_CTRL_DFP_GET_INFO_PARAMS  dfpParams;
     NvU32 dfpFlags;
     dpMemZero(&dfpParams, sizeof(dfpParams));
     dfpParams.subDeviceInstance = subdeviceIndex;
@@ -1516,6 +1515,10 @@ bool EvoMainLink::skipPowerdownEdpPanelWhenHeadDetach()
     return _skipPowerdownEDPPanelWhenHeadDetach;
 }
 
+bool EvoMainLink::isMSTPCONCapsReadDisabled()
+{
+    return _isMSTPCONCapsReadDisabled;
+}
 
 bool EvoMainLink::isActive()
 {
