@@ -238,7 +238,7 @@ NvBool nvDpyIsHdmiDepth30Evo(const NVDpyEvoRec *pDpyEvo)
 /*!
  * Updates the display's HDMI 2.0 capabilities to the RM.
  */
-void nvUpdateHdmiCaps(NVDpyEvoPtr pDpyEvo)
+void nvSendHdmiCapsToRm(NVDpyEvoPtr pDpyEvo)
 {
     NV0073_CTRL_SPECIFIC_SET_HDMI_SINK_CAPS_PARAMS params = { 0 };
     NVParsedEdidEvoPtr pParsedEdid = &pDpyEvo->parsedEdid;
@@ -256,7 +256,7 @@ void nvUpdateHdmiCaps(NVDpyEvoPtr pDpyEvo)
     params.caps = 0;
 
     /*
-     * nvUpdateHdmiCaps() gets called on dpy's connect/disconnect events
+     * nvSendHdmiCapsToRm() gets called on dpy's connect/disconnect events
      * to set/clear capabilities, clear capabilities if parsed edid
      * is not valid.
      */
@@ -1835,15 +1835,11 @@ NvBool nvHdmi204k60HzRGB444Allowed(const NVDpyEvoRec *pDpyEvo,
 void nvHdmiSetVRR(NVDispEvoPtr pDispEvo, NvU32 head, NvBool enable)
 {
     NVT_EXTENDED_METADATA_PACKET_INFOFRAME empInfoFrame;
-    NVT_EXTENDED_METADATA_PACKET_INFOFRAME_CTRL empCtrl;
+    NVT_EXTENDED_METADATA_PACKET_INFOFRAME_CTRL empCtrl = { 0 };
     NVHDMIPKT_TC transmitControl;
     NVT_STATUS status;
 
-    nvkms_memset(&empCtrl, NVT_INFOFRAME_CTRL_DONTCARE,
-                 sizeof(empCtrl));
-
     empCtrl.EnableVRR = enable;
-    empCtrl.EnableQMS = FALSE;
 
     status = NvTiming_ConstructExtendedMetadataPacketInfoframe(&empCtrl,
                                                                &empInfoFrame);
@@ -2401,17 +2397,19 @@ NvBool nvHdmiFrlQueryConfig(
     NvU8 *pHdmiFrlBpc,
     NVDscInfoEvoRec *pDscInfo)
 {
-    // Try first with 10 BPC
-    if (nvHdmiFrlQueryConfigOneBpc(pDpyEvo,
-                                   pModeTimings,
-                                   pHwTimings,
-                                   b2Heads1Or,
-                                   pValidationParams,
-                                   HDMI_BPC10,
-                                   pConfig,
-                                   pHdmiFrlBpc,
-                                   pDscInfo)) {
-        return TRUE;
+    if (nvDpyIsHdmiDepth30Evo(pDpyEvo)) {
+        // Try first with 10 BPC
+        if (nvHdmiFrlQueryConfigOneBpc(pDpyEvo,
+                                       pModeTimings,
+                                       pHwTimings,
+                                       b2Heads1Or,
+                                       pValidationParams,
+                                       HDMI_BPC10,
+                                       pConfig,
+                                       pHdmiFrlBpc,
+                                       pDscInfo)) {
+            return TRUE;
+        }
     }
 
     // Try again with 8 BPC
