@@ -46,6 +46,8 @@
 #include <nv_ref.h>               // NV_PMC_BOOT_1_VGPU
 #include "nvdevid.h"
 
+#include "g_vgpu_chip_flags.h"    // vGPU device names
+
 #define NV_VFIO_PCI_BAR0_REGION_INDEX 0
 #define NV_VFIO_PCI_BAR1_REGION_INDEX 1
 #define NV_VFIO_PCI_BAR2_REGION_INDEX 2
@@ -63,16 +65,6 @@ NvBool hypervisorIsVgxHyper_IMPL(void)
     return os_is_vgx_hyper();
 }
 
-NvBool hypervisorIsAC_IMPL(void)
-{
-    return NV_FALSE;
-}
-
-void hypervisorSetACSupported_IMPL(OBJHYPERVISOR *pHypervisor)
-{
-    pHypervisor->bIsACSupported = NV_TRUE;
-}
-
 NV_STATUS hypervisorInjectInterrupt_IMPL
 (
     OBJHYPERVISOR *pHypervisor,
@@ -82,7 +74,7 @@ NV_STATUS hypervisorInjectInterrupt_IMPL
     NV_STATUS status = NV_ERR_NOT_SUPPORTED;
 
     if (pVgpuNsIntr->pVgpuVfioRef)
-        status = osVgpuInjectInterrupt(pVgpuNsIntr->pVgpuVfioRef);
+        return NV_ERR_NOT_SUPPORTED;
     else
     {
         if (pVgpuNsIntr->guestMSIAddr && pVgpuNsIntr->guestMSIData)
@@ -150,14 +142,22 @@ static NV_STATUS get_available_instances(
 
                 swizzIdInUseMask = kmigmgrGetSwizzIdInUseMask(pGpu, pKernelMIGManager);
 
+                if (!vgpuTypeInfo->gpuInstanceSize)
+                {
+                    // Query for a non MIG vgpuType
+                    NV_PRINTF(LEVEL_INFO, "%s Query for a non MIG vGPU type \n",
+                              __FUNCTION__);
+                    rmStatus = NV_OK;
+                    goto exit;
+                }
+
                 rmStatus = kvgpumgrGetPartitionFlag(vgpuTypeInfo->vgpuTypeId,
                                                    &partitionFlag);
                 if (rmStatus != NV_OK)
                 {
                     // Query for a non MIG vgpuType
-                    NV_PRINTF(LEVEL_ERROR, "%s Query for a non MIG vGPU type \n",
+                    NV_PRINTF(LEVEL_ERROR, "%s failed to get partition flags.\n",
                               __FUNCTION__);
-                    rmStatus = NV_OK;
                     goto exit;
                 }
 
@@ -200,7 +200,7 @@ static NV_STATUS get_available_instances(
                 if (vgpuTypeInfo->gpuInstanceSize)
                 {
                     // Query for a MIG vgpuType
-                    NV_PRINTF(LEVEL_ERROR, "%s Query for a MIG vGPU type \n",
+                    NV_PRINTF(LEVEL_INFO, "%s Query for a MIG vGPU type \n",
                               __FUNCTION__);
                     rmStatus = NV_OK;
                     goto exit;
@@ -208,7 +208,7 @@ static NV_STATUS get_available_instances(
 
                 if (!(pKernelVgpuMgr->pgpuInfo[pgpuIndex].createdVfMask & NVBIT64(fnId)))
                 {
-                    if (kvgpumgrCheckVgpuTypeCreatable(&pKernelVgpuMgr->pgpuInfo[pgpuIndex], vgpuTypeInfo) == NV_OK)
+                    if (kvgpumgrCheckVgpuTypeCreatable(pGpu, &pKernelVgpuMgr->pgpuInfo[pgpuIndex], vgpuTypeInfo) == NV_OK)
                         *avail_instances = 1;
                 }
             }
@@ -216,7 +216,7 @@ static NV_STATUS get_available_instances(
     }
     else
     {
-        if (kvgpumgrCheckVgpuTypeCreatable(&pKernelVgpuMgr->pgpuInfo[pgpuIndex], vgpuTypeInfo) == NV_OK)
+        if (kvgpumgrCheckVgpuTypeCreatable(pGpu, &pKernelVgpuMgr->pgpuInfo[pgpuIndex], vgpuTypeInfo) == NV_OK)
             *avail_instances = vgpuTypeInfo->maxInstance - pKernelVgpuMgr->pgpuInfo[pgpuIndex].numCreatedVgpu;
     }
 
@@ -620,6 +620,16 @@ NV_STATUS NV_API_CALL nv_vgpu_get_bar_info(
             if (status == NV_OK && value)
                 bOverrideBar1Size = NV_TRUE;
         }
+      
+        if (gpuIsVfResizableBAR1Supported(pGpu))
+        {
+            if ((*size > pGpu->sriovState.vfBarSize[1]) ||
+                (!portStringCompare("Compute", (const char *)vgpuTypeInfo->vgpuClass, 7)))
+            {
+                *size = pGpu->sriovState.vfBarSize[1];
+            }
+        } 
+
         if (bOverrideBar1Size) {
             NvU64 bar1SizeInBytes, guestBar1;
             NvU64 gpuBar1LowerLimit = 256 * 1024 * 1024; // bar1 lower limit for override_bar1_length parameter
@@ -856,8 +866,17 @@ NV_STATUS  NV_API_CALL  nv_vgpu_get_sparse_mmap(
                                                           numAreas, NULL, NULL);
                 if (rmStatus == NV_OK)
                 {
-                    os_alloc_mem((void **)&vfRegionOffsets, sizeof(NvU64) * (*numAreas));
-                    os_alloc_mem((void **)&vfRegionSizes, sizeof (NvU64) * (*numAreas));
+                    rmStatus = os_alloc_mem((void **)&vfRegionOffsets, sizeof(NvU64) * (*numAreas));
+                    if (rmStatus != NV_OK)
+                        goto cleanup;
+
+                    rmStatus = os_alloc_mem((void **)&vfRegionSizes, sizeof (NvU64) * (*numAreas));
+                    if (rmStatus != NV_OK)
+                    {
+                        os_free_mem(vfRegionOffsets);
+                        goto cleanup;
+                    }
+
                     if (vfRegionOffsets && vfRegionSizes)
                     {
                         rmStatus = kbifGetVFSparseMmapRegions_HAL(pGpu, pKernelBif, pKernelHostVgpuDevice, os_page_size,
@@ -867,6 +886,11 @@ NV_STATUS  NV_API_CALL  nv_vgpu_get_sparse_mmap(
                             *offsets = vfRegionOffsets;
                             *sizes   = vfRegionSizes;
                         }
+                        else
+                        {
+                            os_free_mem(vfRegionOffsets);
+                            os_free_mem(vfRegionSizes);
+                        }
                     }
                     else
                     {
@@ -916,8 +940,16 @@ NV_STATUS  NV_API_CALL  nv_vgpu_get_sparse_mmap(
                 {
                     NvU32 i = 0;
                     NvU64 *tmpOffset, *tmpSize;
-                    os_alloc_mem((void **)offsets, sizeof(NvU64) * (*numAreas));
-                    os_alloc_mem((void **)sizes, sizeof (NvU64) * (*numAreas));
+                    rmStatus = os_alloc_mem((void **)offsets, sizeof(NvU64) * (*numAreas));
+                    if (rmStatus != NV_OK)
+                        goto cleanup;
+
+                    rmStatus = os_alloc_mem((void **)sizes, sizeof (NvU64) * (*numAreas));
+                    if (rmStatus != NV_OK)
+                    {
+                        os_free_mem(*offsets);
+                        goto cleanup;
+                    }
 
                     tmpOffset = *offsets;
                     tmpSize   = *sizes;
@@ -1141,6 +1173,10 @@ NV_STATUS osIsVfioPciCorePresent(void)
     return os_call_vgpu_vfio((void *) &vgpu_info, CMD_VFIO_PCI_CORE_PRESENT);
 }
 
+NvU32 osGetGridCspSupport(void)
+{
+    return os_get_grid_csp_support();
+}
 
 void initVGXSpecificRegistry(OBJGPU *pGpu)
 {
@@ -1165,12 +1201,14 @@ void initVGXSpecificRegistry(OBJGPU *pGpu)
 
 NV_STATUS rm_is_vgpu_supported_device(
     OS_GPU_INFO *pOsGpuInfo,
-    NvU32       pmc_boot_1
+    NvU32       pmc_boot_1,
+    NvU32       pmc_boot_42
 )
 {
     OBJSYS *pSys = SYS_GET_INSTANCE();
     OBJHYPERVISOR *pHypervisor = SYS_GET_HYPERVISOR(pSys);
     NvBool is_sriov_enabled = FLD_TEST_DRF(_PMC, _BOOT_1, _VGPU, _VF, pmc_boot_1);
+    NvU32     i;
 
     // if not running in vGPU mode (guest VM) return NV_OK
     if (!(pHypervisor && pHypervisor->bIsHVMGuest &&
@@ -1180,5 +1218,28 @@ NV_STATUS rm_is_vgpu_supported_device(
         return NV_OK;
     }
 
-    return NV_OK;
+    if (!is_sriov_enabled)
+    {
+        return NV_OK;
+    }
+
+    if (os_is_grid_supported() && !gpumgrIsVgxRmFirmwareCapableChip(pmc_boot_42))
+    {
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    if (os_is_grid_supported())
+    {
+        for (i = 0; i < NV_ARRAY_ELEMENTS(sVgpuUsmTypes); i++)
+        {
+            if (pOsGpuInfo->pci_info.device_id == sVgpuUsmTypes[i].ulDevID &&
+                    pOsGpuInfo->subsystem_vendor == sVgpuUsmTypes[i].ulSubSystemVendorID &&
+                    pOsGpuInfo->subsystem_id == sVgpuUsmTypes[i].ulSubID)
+            {
+                return NV_OK;
+            }
+        }
+    }
+
+    return NV_ERR_NOT_SUPPORTED;
 }
