@@ -103,6 +103,7 @@ typedef enum
    RM_INIT_GPU_LOAD_FAILED,
    RM_INIT_GPU_UNIVERSAL_VALIDATION_FAILED,
    RM_INIT_GPU_DMA_CONFIGURATION_FAILED,
+   RM_INIT_GPU_GPUMGR_EXPANDED_VISIBILITY_FAILED,
 
    /* vbios errors */
    RM_INIT_VBIOS_FAILED                =  0x30,
@@ -190,7 +191,6 @@ static void        initUnixSpecificRegistry(OBJGPU *);
 NvBool osRmInitRm(OBJOS *pOS)
 {
     OBJSYS    *pSys = SYS_GET_INSTANCE();
-    NvU64 system_memory_size = (NvU64)-1;
 
     NV_PRINTF(LEVEL_INFO, "init rm\n");
 
@@ -274,13 +274,6 @@ NvBool osRmInitRm(OBJOS *pOS)
         return NV_FALSE;
     }
 
-    system_memory_size = NV_RM_PAGES_PER_OS_PAGE * os_get_num_phys_pages();
-
-    // if known, relay the number of system memory pages (in terms of RM page
-    // size) to the RM; this is needed for e.g. TurboCache parts.
-    if (system_memory_size != (NvU64)-1)
-        pOS->SystemMemorySize = system_memory_size;
-
     // Setup any ThreadState defaults
     threadStateInitSetupFlags(THREAD_STATE_SETUP_FLAGS_ENABLED |
                               THREAD_STATE_SETUP_FLAGS_TIMEOUT_ENABLED |
@@ -420,6 +413,126 @@ osHandleGpuLost
     return NV_OK;
 }
 
+/*!
+ * @brief Traverse bus topology till Gpu's root port.
+ * If any of the intermediate bridge has TB3 supported vendorId and hotplug
+ * capability(not necessarily same bridge), mark the Gpu as External Gpu.
+ *
+ * @params[in]    pGpu    OBJGPU pointer
+ * @params[in]    pCl     OBJCL pointer
+ *
+* @return NV_OK
+*      Identified to be eGPU
+* @return others
+ *     Not an eGPU / error on identfying
+ *
+ */
+NvBool
+RmCheckForExternalGpu
+(
+    OBJGPU *pGpu,
+    OBJCL *pCl
+)
+{
+    NvU8 bus;
+    NvU32 domain;
+    void *handleUp;
+    NvU8 busUp, devUp, funcUp;
+    NvU16 vendorIdUp, deviceIdUp;
+    NvU32 portCaps, pciCaps, slotCaps;
+    NvU32 PCIECapPtr;
+    RM_API *pRmApi;
+    NV_STATUS status, rmStatus;
+    NvBool bTb3Bridge = NV_FALSE, bSlotHotPlugSupport = NV_FALSE;
+    NvBool iseGPUBridge = NV_FALSE;
+
+    pRmApi  = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    domain  = gpuGetDomain(pGpu);
+    bus     = gpuGetBus(pGpu);
+    do
+    {
+        // Find the upstream bridge
+        handleUp = clFindP2PBrdg(pCl, domain, bus, &busUp, &devUp, &funcUp, &vendorIdUp, &deviceIdUp);
+        if (!handleUp)
+        {
+            return iseGPUBridge;
+        }
+
+        if (vendorIdUp == PCI_VENDOR_ID_INTEL)
+        {
+            // Check for the supported TB3(ThunderBolt 3) bridges.
+            NV2080_CTRL_INTERNAL_GET_EGPU_BRIDGE_INFO_PARAMS params = { 0 };
+
+            // LOCK: acquire GPUs lock
+            rmStatus = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE,
+                                         RM_LOCK_MODULES_INIT);
+            if (rmStatus != NV_OK)
+            {
+                return iseGPUBridge;
+            }
+            params.pciDeviceId = deviceIdUp;
+            status = pRmApi->Control(pRmApi,
+                                     pGpu->hInternalClient,
+                                     pGpu->hInternalSubdevice,
+                                     NV2080_CTRL_CMD_INTERNAL_GET_EGPU_BRIDGE_INFO,
+                                     &params,
+                                     sizeof(params));
+            // UNLOCK: release GPUs lock
+            rmGpuLocksRelease(GPUS_LOCK_FLAGS_NONE, NULL);
+
+            if (status != NV_OK)
+            {
+                NV_PRINTF(LEVEL_ERROR,
+                          "Error 0x%08x on eGPU Approval for Bridge ID: 0x%08x\n", status, deviceIdUp);
+                DBG_BREAKPOINT();
+                return iseGPUBridge;
+            }
+            else
+            {
+                // Check for the approved eGPU BUS TB3
+                if (params.iseGPUBridge &&
+                    params.approvedBusType == NV2080_CTRL_INTERNAL_EGPU_BUS_TYPE_TB3)
+                {
+                    bTb3Bridge =  NV_TRUE;
+                }
+            }
+        }
+
+        if (NV_OK != clSetPortPcieCapOffset(pCl, handleUp, &PCIECapPtr))
+        {
+            // PCIE bridge but no cap pointer.
+            break;
+        }
+
+        // Get the PCIE capabilities.
+        pciCaps = osPciReadDword(handleUp, CL_PCIE_CAP - CL_PCIE_BEGIN + PCIECapPtr);
+        if (CL_PCIE_CAP_SLOT & pciCaps)
+        {
+            // Get the slot capabilities.
+            slotCaps = osPciReadDword(handleUp, CL_PCIE_SLOT_CAP - CL_PCIE_BEGIN + PCIECapPtr);
+
+            if ((CL_PCIE_SLOT_CAP_HOTPLUG_CAPABLE & slotCaps) &&
+                (CL_PCIE_SLOT_CAP_HOTPLUG_SURPRISE & slotCaps))
+            {
+                bSlotHotPlugSupport = NV_TRUE;
+            }
+        }
+
+        if (bTb3Bridge && bSlotHotPlugSupport)
+        {
+            iseGPUBridge = NV_TRUE;
+            break;
+        }
+
+        bus = busUp;
+
+        // Get port caps to check if PCIE bridge is the root port
+        portCaps = osPciReadDword(handleUp, CL_PCIE_CAP - CL_PCIE_BEGIN + PCIECapPtr);
+
+    } while (!CL_IS_ROOT_PORT(portCaps));
+    return iseGPUBridge;
+}
+
 /*
  * Initialize the required GPU information by doing RMAPI control calls
  * and store the same in the UNIX specific data structures.
@@ -532,6 +645,8 @@ osInitNvMapping(
     nv_priv_t *nvp = NV_GET_NV_PRIV(nv);
     NvU32 deviceInstance;
     NvU32 data = 0;
+    NvU32 dispIsoStreamId;
+    NvU32 dispNisoStreamId;
 
     NV_PRINTF(LEVEL_INFO, "osInitNvMapping:\n");
 
@@ -550,8 +665,6 @@ osInitNvMapping(
     {
         NV_PRINTF(LEVEL_ERROR, "*** cannot allocate GPU lock\n");
         RM_SET_ERROR(*status, RM_INIT_GPU_GPUMGR_ALLOC_GPU_FAILED);
-        // RM_BASIC_LOCK_MODEL: free GPU lock
-        rmGpuLockFree(*pDeviceReference);
         return;
     }
 
@@ -610,6 +723,7 @@ osInitNvMapping(
         gpuAttachArg->instLength      = nv->bars[NV_GPU_BAR_INDEX_IMEM].size;
 
         gpuAttachArg->iovaspaceId     = nv->iovaspace_id;
+        gpuAttachArg->cpuNumaNodeId   = nv->cpu_numa_node_id;
     }
 
     //
@@ -703,6 +817,17 @@ osInitNvMapping(
 
         nv->preserve_vidmem_allocations = NV_TRUE;
     }
+
+    // Check if SMMU can be enabled on PushBuffer Aperture
+    nv_get_disp_smmu_stream_ids(nv, &dispIsoStreamId, &dispNisoStreamId);
+    if (dispNisoStreamId != NV_U32_MAX)
+    {
+        pGpu->setProperty(pGpu, PDB_PROP_GPU_DISP_PB_REQUIRES_SMMU_BYPASS, NV_FALSE);
+    }
+    else
+    {
+        pGpu->setProperty(pGpu, PDB_PROP_GPU_DISP_PB_REQUIRES_SMMU_BYPASS, NV_TRUE);
+    }
 }
 
 void osInitScalabilityOptions
@@ -746,21 +871,6 @@ osTeardownScalability(
     return clTeardownPcie(pGpu, pCl);
 }
 
-static void
-populateDeviceAttributes(
-    OBJGPU  *pGpu,
-    nv_state_t *nv
-)
-{
-    OBJSYS *pSys = SYS_GET_INSTANCE();
-    OBJCL  *pCl  = SYS_GET_CL(pSys);
-
-    if ((pCl != NULL) && pCl->getProperty(pCl, PDB_PROP_CL_IS_EXTERNAL_GPU))
-    {
-        nv->is_external_gpu = NV_TRUE;
-    }
-}
-
 static void
 RmSetConsolePreservationParams(OBJGPU *pGpu)
 {
@@ -883,6 +993,8 @@ RmInitNvDevice(
 {
     // set the device context
     OBJGPU *pGpu = gpumgrGetGpu(deviceReference);
+    OBJSYS *pSys = SYS_GET_INSTANCE();
+    OBJCL  *pCl  = SYS_GET_CL(pSys);
     nv_state_t *nv = NV_GET_NV_STATE(pGpu);
     nv_priv_t *nvp = NV_GET_NV_PRIV(nv);
 
@@ -900,14 +1012,17 @@ RmInitNvDevice(
         return;
     }
 
-    os_disable_console_access();
-
+    // Configure eGPU setting
+    if (RmCheckForExternalGpu(pGpu, pCl))
+    {
+        pCl->setProperty(pCl, PDB_PROP_CL_IS_EXTERNAL_GPU, NV_TRUE);
+        nv->is_external_gpu = NV_TRUE;
+    }
     status->rmStatus = gpumgrStateInitGpu(pGpu);
     if (status->rmStatus != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "*** Cannot initialize the device\n");
         RM_SET_ERROR(*status, RM_INIT_GPU_STATE_INIT_FAILED);
-        os_enable_console_access();
         return;
     }
     nvp->flags |= NV_INIT_FLAG_GPU_STATE;
@@ -937,13 +1052,10 @@ RmInitNvDevice(
         NV_PRINTF(LEVEL_ERROR,
                   "*** Cannot load state into the device\n");
         RM_SET_ERROR(*status, RM_INIT_GPU_LOAD_FAILED);
-        os_enable_console_access();
         return;
     }
     nvp->flags |= NV_INIT_FLAG_GPU_STATE_LOAD;
 
-    os_enable_console_access();
-
     status->rmStatus = gpuPerformUniversalValidation_HAL(pGpu);
     if (status->rmStatus != NV_OK)
     {
@@ -1160,6 +1272,7 @@ NvBool RmInitPrivateState(
     nv_priv_t *nvp;
     NvU32 gpuId;
     NvU32 pmc_boot_0 = 0;
+    NvU32 pmc_boot_1 = 0;
     NvU32 pmc_boot_42 = 0;
 
     NV_SET_NV_PRIV(pNv, NULL);
@@ -1177,6 +1290,7 @@ NvBool RmInitPrivateState(
         }
 
         pmc_boot_0 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_0);
+        pmc_boot_1 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_1);
         pmc_boot_42 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_42);
 
         os_unmap_kernel_space(pNv->regs->map_u, os_page_size);
@@ -1204,6 +1318,7 @@ NvBool RmInitPrivateState(
 
     pNv->iovaspace_id = nv_requires_dma_remap(pNv) ? gpuId :
                                                      NV_IOVA_DOMAIN_NONE;
+    pNv->cpu_numa_node_id = NV0000_CTRL_NO_NUMA_NODE;
 
     kvgpumgrAttachGpu(pNv->gpu_id);
 
@@ -1216,6 +1331,7 @@ NvBool RmInitPrivateState(
     os_mem_set(nvp, 0, sizeof(*nvp));
     nvp->status = NV_ERR_INVALID_STATE;
     nvp->pmc_boot_0 = pmc_boot_0;
+    nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
     NV_SET_NV_PRIV(pNv, nvp);
 
@@ -1234,7 +1350,7 @@ void RmClearPrivateState(
     nv_i2c_adapter_entry_t i2c_adapters[MAX_I2C_ADAPTERS];
     nv_dynamic_power_t dynamicPowerCopy;
     NvU32 x = 0;
-    NvU32 pmc_boot_0, pmc_boot_42;
+    NvU32 pmc_boot_0, pmc_boot_1, pmc_boot_42;
 
     //
     // Do not clear private state after GPU resets, it is used while
@@ -1252,6 +1368,7 @@ void RmClearPrivateState(
     pRegistryCopy = nvp->pRegistry;
     dynamicPowerCopy = nvp->dynamic_power;
     pmc_boot_0 = nvp->pmc_boot_0;
+    pmc_boot_1 = nvp->pmc_boot_1;
     pmc_boot_42 = nvp->pmc_boot_42;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
@@ -1267,6 +1384,7 @@ void RmClearPrivateState(
     nvp->pRegistry = pRegistryCopy;
     nvp->dynamic_power = dynamicPowerCopy;
     nvp->pmc_boot_0 = pmc_boot_0;
+    nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
@@ -1537,6 +1655,7 @@ NvBool RmInitAdapter(
     KernelDisplay  *pKernelDisplay;
     const void     *gspFwHandle = NULL;
     const void     *gspFwLogHandle = NULL;
+    NvBool          consoleDisabled = NV_FALSE;
 
     GSP_FIRMWARE    gspFw = {0};
     PORT_UNREFERENCED_VARIABLE(gspFw);
@@ -1580,6 +1699,17 @@ NvBool RmInitAdapter(
         }
     }
 
+    //
+    // Initialization path requires expanded GPU visibility in GPUMGR
+    // in order to access the GPU undergoing initialization.
+    //
+    status.rmStatus = gpumgrThreadEnableExpandedGpuVisibility();
+    if (status.rmStatus != NV_OK)
+    {
+        RM_SET_ERROR(status, RM_INIT_GPU_GPUMGR_EXPANDED_VISIBILITY_FAILED);
+        goto shutdown;
+    }
+
     // initialize the RM device register mapping
     osInitNvMapping(nv, &devicereference, &status);
     if (! RM_INIT_SUCCESS(status.initStatus) )
@@ -1616,7 +1746,7 @@ NvBool RmInitAdapter(
     KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
     if ((pKernelFsp != NULL) && !IS_GSP_CLIENT(pGpu) && !IS_VIRTUAL(pGpu))
     {
-        status.rmStatus = kfspSendBootCommands_HAL(pGpu, pKernelFsp);
+        status.rmStatus = kfspPrepareAndSendBootCommands_HAL(pGpu, pKernelFsp);
         if (status.rmStatus != NV_OK)
         {
             NV_PRINTF(LEVEL_ERROR, "FSP boot command failed.\n");
@@ -1629,6 +1759,16 @@ NvBool RmInitAdapter(
 
     RmInitAcpiMethods(pOS, pSys, pGpu);
 
+    //
+    // For GPU driving console, disable console access here, to ensure no console
+    // writes through BAR1 can interfere with physical RM's setup of BAR1
+    //
+    if (rm_get_uefi_console_status(nv))
+    {
+        os_disable_console_access();
+        consoleDisabled = NV_TRUE;
+    }
+
     //
     // If GSP fw RM support is enabled then start the GSP microcode
     // (including the task running the full instance of the RM) and
@@ -1668,10 +1808,8 @@ NvBool RmInitAdapter(
     if (IS_PASSTHRU(pGpu))
         nv->flags |= NV_FLAG_PASSTHRU;
 
-    populateDeviceAttributes(pGpu, nv);
-
     initVendorSpecificRegistry(pGpu, nv->pci_info.device_id);
-    if (!IS_VIRTUAL(pGpu) && !IS_GSP_CLIENT(pGpu))
+    if (!IS_VIRTUAL(pGpu))
     {
         initNbsiTable(pGpu);
     }
@@ -1704,6 +1842,18 @@ NvBool RmInitAdapter(
         goto shutdown;
     }
 
+    if (consoleDisabled)
+    {
+        os_enable_console_access();
+        consoleDisabled = NV_FALSE;
+    }
+
+    //
+    // Expanded GPU visibility in GPUMGR is no longer needed once the
+    // GPU is initialized.
+    //
+    gpumgrThreadDisableExpandedGpuVisibility();
+
     // LOCK: acquire GPUs lock
     status.rmStatus = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE,
                                         RM_LOCK_MODULES_INIT);
@@ -1867,6 +2017,13 @@ NvBool RmInitAdapter(
  shutdown:
     nv->flags &= ~NV_FLAG_IN_RECOVERY;
 
+    gpumgrThreadDisableExpandedGpuVisibility();
+
+    if (consoleDisabled)
+    {
+        os_enable_console_access();
+    }
+
     // call ShutdownAdapter to undo anything we've done above
     RmShutdownAdapter(nv);
 
@@ -1903,6 +2060,14 @@ void RmShutdownAdapter(
         // LOCK: acquire GPUs lock
         if (rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE, RM_LOCK_MODULES_DESTROY) == NV_OK)
         {
+            //
+            // Shutdown path requires expanded GPU visibility in GPUMGR in order
+            // to access the GPU undergoing shutdown which may not be fully
+            // initialized, and to continue accessing the GPU undergoing shutdown
+            // after state destroy.
+            //
+            NV_ASSERT_OK(gpumgrThreadEnableExpandedGpuVisibility());
+
             RmDestroyDeferredDynamicPowerManagement(nv);
 
             freeNbsiTable(pGpu);
@@ -1952,6 +2117,12 @@ void RmShutdownAdapter(
             gpumgrDetachGpu(gpuInstance);
             gpumgrDestroyDevice(deviceInstance);
 
+            //
+            // Expanded GPU visibility in GPUMGR is no longer needed once the
+            // GPU is removed from GPUMGR.
+            //
+            gpumgrThreadDisableExpandedGpuVisibility();
+
             if (nvp->flags & NV_INIT_FLAG_DMA)
             {
                 RmTeardownDeviceDma(nv);
