@@ -136,15 +136,13 @@ kfspStateInitUnlocked_IMPL
 }
 
 /*!
- * @brief Destroy FSP state
+ * @brief Clean up objects used when sending GSP-FMC and FRTS info to FSP
  *
  * @param[in]  pGpu        GPU object pointer
  * @param[in]  pKernelFsp  FSP object pointer
- *
- * @return 'NV_OK' if the FSP state was successfully destroyed
  */
 void
-kfspStateDestroy_IMPL
+kfspCleanupBootState_IMPL
 (
     OBJGPU    *pGpu,
     KernelFsp *pKernelFsp
@@ -155,6 +153,7 @@ kfspStateDestroy_IMPL
 
     if (pKernelFsp->pSysmemFrtsMemdesc != NULL)
     {
+        kfspFrtsSysmemLocationClear_HAL(pGpu, pKernelFsp);
         memdescUnmap(pKernelFsp->pSysmemFrtsMemdesc, NV_TRUE, 0,
             memdescGetKernelMapping(pKernelFsp->pSysmemFrtsMemdesc),
             memdescGetKernelMappingPriv(pKernelFsp->pSysmemFrtsMemdesc));
@@ -163,13 +162,6 @@ kfspStateDestroy_IMPL
         pKernelFsp->pSysmemFrtsMemdesc = NULL;
     }
 
-    if (pKernelFsp->pVidmemFrtsMemdesc != NULL)
-    {
-        memdescFree(pKernelFsp->pVidmemFrtsMemdesc);
-        memdescDestroy(pKernelFsp->pVidmemFrtsMemdesc);
-        pKernelFsp->pVidmemFrtsMemdesc = NULL;
-    }
-
     if (pKernelFsp->pGspFmcMemdesc != NULL)
     {
         memdescFree(pKernelFsp->pGspFmcMemdesc);
@@ -186,13 +178,34 @@ kfspStateDestroy_IMPL
 
 }
 
+/*!
+ * @brief Destroy FSP state
+ *
+ * @param[in]  pGpu        GPU object pointer
+ * @param[in]  pKernelFsp  FSP object pointer
+ */
+void
+kfspStateDestroy_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelFsp *pKernelFsp
+)
+{
+    kfspCleanupBootState(pGpu, pKernelFsp);
+
+    if (pKernelFsp->pVidmemFrtsMemdesc != NULL)
+    {
+        memdescFree(pKernelFsp->pVidmemFrtsMemdesc);
+        memdescDestroy(pKernelFsp->pVidmemFrtsMemdesc);
+        pKernelFsp->pVidmemFrtsMemdesc = NULL;
+    }
+}
+
 /*!
  * @brief Override default behaviour of reset
  *
  * @param[in]  pGpu       GPU object pointer
  * @param[in]  pKernelFsp FSP object pointer
- *
- * @return
  */
 void
 kfspSecureReset_IMPL
@@ -246,32 +259,50 @@ kfspPollForQueueEmpty_IMPL
     KernelFsp *pKernelFsp
 )
 {
+    NV_STATUS status = NV_OK;
     RMTIMEOUT timeout;
 
-    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, GPU_TIMEOUT_FLAGS_OSTIMER | GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE);
+    gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout,
+        GPU_TIMEOUT_FLAGS_OSTIMER |
+        GPU_TIMEOUT_FLAGS_BYPASS_THREAD_STATE);
 
     while (!kfspIsQueueEmpty(pGpu, pKernelFsp))
     {
         //
-        // For now we assume that any response from FSP before RM message send is complete
-        // indicates an error and we should abort.
+        // For now we assume that any response from FSP before RM message
+        // send is complete indicates an error and we should abort.
+        //
+        // Ongoing dicussion on usefullness of this check. Bug to be filed.
         //
         if (!kfspIsMsgQueueEmpty(pGpu, pKernelFsp))
         {
             kfspReadMessage(pGpu, pKernelFsp, NULL, 0);
-            NV_PRINTF(LEVEL_ERROR, "Received error message from FSP while waiting for CMDQ to be empty.\n");
-            return NV_ERR_GENERIC;
+            NV_PRINTF(LEVEL_ERROR,
+                "Received error message from FSP while waiting for CMDQ to be empty.\n");
+            status = NV_ERR_GENERIC;
+            break;
         }
 
-        if (gpuCheckTimeout(pGpu, &timeout) == NV_ERR_TIMEOUT)
-        {
-            NV_PRINTF(LEVEL_ERROR, "Timed out waiting for FSP command queue to be empty.\n");
-            return NV_ERR_TIMEOUT;
-        }
         osSpinLoop();
+
+        status = gpuCheckTimeout(pGpu, &timeout);
+        if (status != NV_OK)
+        {
+            if ((status == NV_ERR_TIMEOUT) &&
+                kfspIsQueueEmpty(pGpu, pKernelFsp))
+            {
+                status = NV_OK;
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_ERROR,
+                    "Timed out waiting for FSP command queue to be empty.\n");
+            }
+            break;
+        }
     }
 
-    return NV_OK;
+    return status;
 }
 
 /*!
