@@ -476,24 +476,6 @@ void GspMsgQueuesCleanup(MESSAGE_QUEUE_COLLECTION **ppMQCollection)
     *ppMQCollection = NULL;
 }
 
-/*!
- * Calculate 32-bit checksum
- *
- * This routine assumes that the data is padded out with zeros to the next
- * 8-byte alignment, and it is OK to read past the end to the 8-byte alignment.
- */
-static NV_INLINE NvU32 _checkSum32(void *pData, NvU32 uLen)
-{
-    NvU64 *p        = (NvU64 *)pData;
-    NvU64 *pEnd     = (NvU64 *)((NvUPtr)pData + uLen);
-    NvU64  checkSum = 0;
-
-    while (p < pEnd)
-        checkSum ^= *p++;
-
-    return NvU64_HI32(checkSum) ^ NvU64_LO32(checkSum);
-}
-
 /*!
  * GspMsgQueueSendCommand
  *
@@ -512,7 +494,6 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
     NvU8      *pNextElement     = NULL;
     int        nRet;
     NvU32      i;
-    NvU32      nRetries;
     RMTIMEOUT  timeout;
     NV_STATUS  nvStatus         = NV_OK;
     NvU32      uElementSize     = GSP_MSG_QUEUE_ELEMENT_HDR_SIZE +
@@ -533,14 +514,16 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
 
     pCQE->seqNum    = pMQI->txSeqNum;
     pCQE->elemCount = GSP_MSG_QUEUE_BYTES_TO_ELEMENTS(uElementSize);
-    pCQE->checkSum  = 0;
+    pCQE->checkSum  = 0; // The checkSum field is included in the checksum calculation, so zero it.
 
-    ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
-    if (pCC != NULL && pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_ENCRYPT_ENABLED))
+    if (gpuIsCCFeatureEnabled(pGpu))
     {
+        ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
+
         // Use sequence number as AAD.
         portMemCopy((NvU8*)pCQE->aadBuffer, sizeof(pCQE->aadBuffer), (NvU8 *)&pCQE->seqNum, sizeof(pCQE->seqNum));
 
+        // We need to encrypt the full queue elements to obscure the data.
         nvStatus = ccslEncrypt(pCC->pRpcCcslCtx,
                                (pCQE->elemCount * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN) - GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
                                pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
@@ -549,15 +532,26 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
                                pSrc + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
                                pCQE->authTagBuffer);
 
-        if(nvStatus != NV_OK)
+        if (nvStatus != NV_OK)
         {
+            // Do not re-try if encryption fails.
             NV_PRINTF(LEVEL_ERROR, "Encryption failed with status = 0x%x.\n", nvStatus);
-            // Do not re-try if decryption failed.
+            if (nvStatus == NV_ERR_INSUFFICIENT_RESOURCES)
+            {
+                // We hit potential IV overflow, this is fatal.
+                NV_PRINTF(LEVEL_ERROR, "Fatal error detected in RPC encrypt: IV overflow!\n");
+                confComputeSetErrorState(pGpu, pCC);
+            }
             return nvStatus;
         }
-    }
 
-    pCQE->checkSum  = _checkSum32(pSrc, pCQE->elemCount * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN);
+        // Now that encryption covers elements completely, include them in checksum.
+        pCQE->checkSum = _checkSum32(pSrc, pCQE->elemCount * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN);
+    }
+    else
+    {
+        pCQE->checkSum = _checkSum32(pSrc, uElementSize);
+    }
 
     for (i = 0; i < pCQE->elemCount; i++)
     {
@@ -569,8 +563,8 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
         // Set a timeout of 1 sec
         gpuSetTimeout(pGpu, 1000000, &timeout, timeoutFlags);
 
-        // Wait for space to put the next element. Retry for up to 10 ms.
-        for (nRetries = 0; ; nRetries++)
+        // Wait for space to put the next element.
+        while (NV_TRUE)
         {
             // Must get the buffers one at a time, since they could wrap.
             pNextElement = (NvU8 *)msgqTxGetWriteBuffer(pMQI->hQueue, i);
@@ -589,7 +583,9 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
         if (pNextElement == NULL)
         {
             pMQI->txBufferFull++;
-            NV_PRINTF_COND(pMQI->txBufferFull == 1, LEVEL_ERROR, LEVEL_INFO, "buffer is full\n");
+            NV_PRINTF_COND(pMQI->txBufferFull == 1, LEVEL_ERROR, LEVEL_INFO,
+                           "buffer is full (waiting for %d free elements, got %d)\n",
+                           pCQE->elemCount, i);
             nvStatus = NV_ERR_BUSY_RETRY;
             goto done;
         }
@@ -652,7 +648,8 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
     NvU32       nRetries;
     NvU32       nMaxRetries  = 3;
     NvU32       nElements    = 1;  // Assume record fits in one queue element for now.
-    NvU32       uElementSize = 0;
+    NvU32       uElementSize;
+    NvU32       checkSum;
     NvU32       seqMismatchDiff = NV_U32_MAX;
     NV_STATUS   nvStatus     = NV_OK;
 
@@ -701,7 +698,25 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
             continue;
 
         // Retry if checksum fails.
-        if (_checkSum32(pMQI->pCmdQueueElement, (nElements * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN)) != 0)
+        if (gpuIsCCFeatureEnabled(pGpu))
+        {
+            //
+            // In the Confidential Compute scenario, the actual message length
+            // is inside the encrypted payload, and we can't access it before
+            // decryption, therefore the checksum encompasses the whole element
+            // range. This makes checksum verification significantly slower
+            // because messages are typically much smaller than element size.
+            //
+            checkSum = _checkSum32(pMQI->pCmdQueueElement,
+                                   (nElements * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN));
+        } else
+        {
+            checkSum = _checkSum32(pMQI->pCmdQueueElement,
+                                   (GSP_MSG_QUEUE_ELEMENT_HDR_SIZE +
+                                    pMQI->pCmdQueueElement->rpc.length));
+        }
+
+        if (checkSum != 0)
         {
             NV_PRINTF(LEVEL_ERROR, "Bad checksum.\n");
             nvStatus = NV_ERR_INVALID_DATA;
@@ -754,9 +769,9 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
         }
     }
 
-    ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
-    if (pCC != NULL && pCC->getProperty(pCC, PDB_PROP_CONFCOMPUTE_ENCRYPT_READY))
+    if (gpuIsCCFeatureEnabled(pGpu))
     {
+        ConfidentialCompute *pCC = GPU_GET_CONF_COMPUTE(pGpu);
         nvStatus = ccslDecrypt(pCC->pRpcCcslCtx,
                                (nElements * GSP_MSG_QUEUE_ELEMENT_SIZE_MIN) - GSP_MSG_QUEUE_ELEMENT_HDR_SIZE,
                                ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
@@ -766,10 +781,11 @@ NV_STATUS GspMsgQueueReceiveStatus(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
                                ((NvU8*)pMQI->pCmdQueueElement) + GSP_MSG_QUEUE_ELEMENT_HDR_SIZE, 
                                ((NvU8*)pMQI->pCmdQueueElement->authTagBuffer));
 
-        if(nvStatus != NV_OK)
+        if (nvStatus != NV_OK)
         {
-            NV_PRINTF(LEVEL_ERROR, "Decryption failed with status = 0x%x.\n", nvStatus);
-            // Do not re-try if decryption failed.
+            // Do not re-try if decryption failed. Decryption failure is considered fatal.
+            NV_PRINTF(LEVEL_ERROR, "Fatal error detected in RPC decrypt: 0x%x!\n", nvStatus);
+            confComputeSetErrorState(pGpu, pCC);
             return nvStatus;
         }
     }
