@@ -31,6 +31,7 @@
 #include "kernel/gpu/gr/kernel_graphics.h"
 #include "kernel/gpu/intr/engine_idx.h"
 #include "kernel/gpu/intr/intr_service.h"
+#include "kernel/gpu/gsp/gsp_trace_rats_macro.h"
 #include "gpu/mmu/kern_gmmu.h"
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
 #include "os/os.h"
@@ -38,6 +39,7 @@
 #include "vgpu/rpc.h"
 #include "virtualization/hypervisor/hypervisor.h"
 #include "gpu/gsp/kernel_gsp.h"
+#include "platform/sli/sli.h"
 
 #include "nv_ref.h"
 #include "nvRmReg.h"
@@ -166,6 +168,16 @@ subdeviceCtrlCmdMcServiceInterrupts_IMPL
         RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
         NV_STATUS status = NV_OK;
 
+        //
+        // Force kernel-RM to service interrupts from GSP-RM. This will allow
+        // kernel-RM to write notifiers and send an ack back to GSP. 
+        // GSP waits for this ack before clearing fast path POSSIBLE_ERR interrupt.
+        //
+        if (pGpu->getProperty(pGpu, PDB_PROP_GPU_FASTPATH_SEQ_ENABLED))
+        {
+            intrServiceStallSingle_HAL(pGpu, pIntr, MC_ENGINE_IDX_GSP, NV_TRUE);
+        }
+
         NV_RM_RPC_CONTROL(pGpu, pRmCtrlParams->hClient, pRmCtrlParams->hObject, pRmCtrlParams->cmd,
                           pRmCtrlParams->pParams, pRmCtrlParams->paramsSize, status);
         if (status != NV_OK)
@@ -524,6 +536,43 @@ intrServiceStallListDevice_IMPL
     SLI_LOOP_END;
 }
 
+
+INTR_TABLE_ENTRY *
+intrGetInterruptTableEntryFromEngineId_IMPL
+(
+    OBJGPU *pGpu,
+    Intr   *pIntr,
+    NvU16   mcEngineId,
+    NvBool  bNonStall
+)
+{
+    InterruptTable    *pIntrTable;
+    InterruptTableIter iter;
+    NV_STATUS          status;
+
+    status = intrGetInterruptTable_HAL(pGpu, pIntr, &pIntrTable);
+    if (status != NV_OK)
+    {
+        NV_ASSERT_OK_FAILED("Failed to get interrupt table", status);
+        return NULL;
+    }
+
+    for (iter = vectIterAll(pIntrTable); vectIterNext(&iter);)
+    {
+        INTR_TABLE_ENTRY *pEntry = iter.pValue;
+        if (pEntry->mcEngine == mcEngineId)
+        {
+            return pEntry;
+        }
+    }
+
+    NV_PRINTF(LEVEL_ERROR,
+              "Could not find the specified engine Id %u\n",
+              mcEngineId);
+    DBG_BREAKPOINT();
+    return NULL;
+}
+
 /*!
  * @brief Get the interrupt vector for the given engine
  *
@@ -537,37 +586,22 @@ intrServiceStallListDevice_IMPL
 NvU32
 intrGetVectorFromEngineId_IMPL
 (
-    OBJGPU   *pGpu,
-    Intr     *pIntr,
-    NvU16     mcEngineId,
-    NvBool    bNonStall
+    OBJGPU *pGpu,
+    Intr   *pIntr,
+    NvU16   mcEngineId,
+    NvBool  bNonStall
 )
 {
-    InterruptTable    *pIntrTable;
-    InterruptTableIter iter;
-    NV_STATUS          status;
+    INTR_TABLE_ENTRY *pEntry = intrGetInterruptTableEntryFromEngineId(pGpu,
+        pIntr,
+        mcEngineId,
+        bNonStall);
 
-    status = intrGetInterruptTable_HAL(pGpu, pIntr, &pIntrTable);
-    if (status != NV_OK)
-    {
-        NV_ASSERT_OK_FAILED("Failed to get interrupt table", status);
-        return NV_INTR_VECTOR_INVALID;
-    }
-
-    for (iter = vectIterAll(pIntrTable); vectIterNext(&iter);)
-    {
-        INTR_TABLE_ENTRY *pEntry = iter.pValue;
-        if (pEntry->mcEngine == mcEngineId)
-        {
-            return bNonStall ? pEntry->intrVectorNonStall : pEntry->intrVector;
-        }
-    }
-
-    NV_PRINTF(LEVEL_ERROR, "Could not find the specified engine Id %u\n", mcEngineId);
-    DBG_BREAKPOINT();
-    return NV_INTR_VECTOR_INVALID;
+    NV_ASSERT_OR_RETURN(pEntry != NULL, NV_INTR_VECTOR_INVALID);
+    return bNonStall ? pEntry->intrVectorNonStall : pEntry->intrVector;
 }
 
+
 /*!
  * @brief Convert a general MC_ENGINE_BITVECTOR to its corresponding hardware
  *        interrupt mask in PMC.
@@ -721,11 +755,15 @@ intrConstructEngine_IMPL
     pIntr->dpcQueue.pRear  = NULL;
     pIntr->bDpcStarted     = NV_FALSE;
 
-    if (!RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM)
+    if (!RMCFG_FEATURE_PLATFORM_WINDOWS)
     {
         pIntr->setProperty(pIntr, PDB_PROP_INTR_DISABLE_PER_INTR_DPC_QUEUEING, NV_TRUE);
     }
 
+    NV_ASSERT_OK_OR_RETURN(vectInit(&pIntr->intrTable,
+                                    portMemAllocatorGetGlobalNonPaged(),
+                                    0 /* capacity */));
+
     return NV_OK;
 }
 
@@ -746,6 +784,8 @@ intrDestruct_IMPL
         portMemFree(pNode);
     }
 
+    vectDestroy(&pIntr->intrTable);
+
 }
 
 
@@ -813,7 +853,7 @@ intrStateInitLocked_IMPL
     if (pIntr->getProperty(pIntr, PDB_PROP_INTR_USE_INTR_MASK_FOR_LOCKING))
     {
         intrGetIntrMask_HAL(pGpu, pIntr, &pIntr->intrMask.engMaskOrig, NULL /* threadstate */);
-        if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM)
+        if (RMCFG_FEATURE_PLATFORM_WINDOWS)
         {
             MC_ENGINE_BITVECTOR engines;
 
@@ -911,7 +951,7 @@ _intrInitRegistryOverrides
     }
 
     pIntr->setProperty(pIntr, PDB_PROP_INTR_USE_INTR_MASK_FOR_LOCKING, NV_FALSE);
-    if (RMCFG_FEATURE_PLATFORM_WINDOWS_LDDM || hypervisorIsVgxHyper())
+    if (RMCFG_FEATURE_PLATFORM_WINDOWS || hypervisorIsVgxHyper())
     {
         // Enable IntrMask Locking by default if supported
         if (pIntr->getProperty(pIntr, PDB_PROP_INTR_MASK_SUPPORTED) &&
@@ -976,7 +1016,12 @@ intrInitInterruptTable_KERNEL
     NvU32     i;
     NV2080_CTRL_INTERNAL_INTR_GET_KERNEL_TABLE_PARAMS *pParams;
 
-    NV_ASSERT_OR_RETURN(vectIsEmpty(&pIntr->intrTable), NV_ERR_INVALID_STATE);
+    //
+    // Unconditionally clear it. For _KERNEL, this function can be called to
+    // re-fetch the KernelRM interrupt table every time PhysicalRM updates it
+    // like after MIG partition configuration changes.
+    //
+    vectClear(&pIntr->intrTable);
 
     pParams = portMemAllocNonPaged(sizeof(*pParams));
     NV_ASSERT_TRUE_OR_GOTO(status, pParams != NULL, NV_ERR_NO_MEMORY, exit);
@@ -996,10 +1041,8 @@ intrInitInterruptTable_KERNEL
                            exit);
 
     NV_ASSERT_OK_OR_GOTO(status,
-        vectInit(&pIntr->intrTable,
-                 portMemAllocatorGetGlobalNonPaged(),
-                 pParams->tableLen),
-        exit);
+                         vectReserve(&pIntr->intrTable, pParams->tableLen),
+                         exit);
     for (i = 0; i < pParams->tableLen; ++i)
     {
         INTR_TABLE_ENTRY entry = {0};
@@ -1049,7 +1092,7 @@ _intrInitServiceTable
 NvU32 intrServiceInterruptRecords_IMPL
 (
     OBJGPU  *pGpu,
-    Intr *pIntr,
+    Intr    *pIntr,
     NvU16    engineIdx,
     NvBool  *pServiced
 )
@@ -1078,7 +1121,10 @@ NvU32 intrServiceInterruptRecords_IMPL
 
     if (bShouldService)
     {
+        GSP_TRACE_RATS_ADD_RECORD(NV_RATS_GSP_TRACE_TYPE_INTR_INFO, pGpu, (NvU32) engineIdx);
+        GSP_TRACE_RATS_ADD_RECORD(NV_RATS_GSP_TRACE_TYPE_INTR_START, pGpu, 0);
         ret = intrservServiceInterrupt(pGpu, pIntrService, &serviceParams);
+        GSP_TRACE_RATS_ADD_RECORD(NV_RATS_GSP_TRACE_TYPE_INTR_END, pGpu, 0);
     }
     return ret;
 }
@@ -1140,14 +1186,16 @@ NV_STATUS intrCheckFecsEventbufferPending_IMPL
 )
 {
     NvU8 i;
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    NvS16 fecsCtxswLogConsumerCount= fecsGetCtxswLogConsumerCount(pGpu, pKernelGraphicsManager);
 
     NV_ASSERT_OR_RETURN(pbCtxswLog != NULL, NV_ERR_INVALID_ARGUMENT);
 
     *pbCtxswLog = NV_FALSE;
 
-    if (pGpu->fecsCtxswLogConsumerCount <= 0)
+    if (fecsCtxswLogConsumerCount <= 0)
     {
-        NV_ASSERT(pGpu->fecsCtxswLogConsumerCount == 0);
+        NV_ASSERT(fecsCtxswLogConsumerCount == 0);
         return NV_OK;
     }
 
@@ -1180,14 +1228,16 @@ intrCheckAndServiceFecsEventbuffer_IMPL
     THREAD_STATE_NODE *pThreadState
 )
 {
+    KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+    NvS16 fecsCtxswLogConsumerCount= fecsGetCtxswLogConsumerCount(pGpu, pKernelGraphicsManager);
     NvU8 i;
 
     if (bitVectorTestAllCleared(pIntrPending))
         return NV_OK;
 
-    if (pGpu->fecsCtxswLogConsumerCount <= 0)
+    if (fecsCtxswLogConsumerCount <= 0)
     {
-        NV_ASSERT(pGpu->fecsCtxswLogConsumerCount == 0);
+        NV_ASSERT(fecsCtxswLogConsumerCount == 0);
         return NV_OK;
     }
 
@@ -1235,7 +1285,8 @@ intrDestroyInterruptTable_IMPL
         pIntr->vectorToMcIdx[tree]       = NULL;
         pIntr->vectorToMcIdxCounts[tree] = 0;
     }
-    vectDestroy(&pIntr->intrTable);
+
+    vectClear(&pIntr->intrTable);
     return NV_OK;
 }
 
@@ -1407,14 +1458,11 @@ _intrServiceStallExactList
 )
 {
     NV_STATUS status;
-
     NvU32  engineIdx;
     NvU32  intr;
     NvBool bHandled;
     NvBool bIntrStuck = NV_FALSE;
     NvBool bPending   = NV_FALSE;
-    NvBool bRequiresPossibleErrorNotifier;
-
     InterruptTable    *pIntrTable;
     InterruptTableIter iter;
 
@@ -1427,20 +1475,6 @@ _intrServiceStallExactList
                          intrGetInterruptTable_HAL(pGpu, pIntr, &pIntrTable),
                          return NV_FALSE);
 
-    bRequiresPossibleErrorNotifier = intrRequiresPossibleErrorNotifier_HAL(pGpu, pIntr, pEngines);
-
-    if (bRequiresPossibleErrorNotifier)
-    {
-        //
-        // Notify CUDA there may be an error in ERR_CONT that they may miss because we're
-        // about to clear it out of the NV_CTRL tree backing ERR_CONT before the interrupt
-        // is serviced.
-        //
-        // info32 contains shadowed value of ERR_CONT
-        //
-        gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_POSSIBLE_ERROR, NULL, 0, intrReadErrCont_HAL(pGpu, pIntr), 0);
-    }
-
     for (iter = vectIterAll(pIntrTable); vectIterNext(&iter);)
     {
         INTR_TABLE_ENTRY *pEntry = iter.pValue;
@@ -1491,18 +1525,6 @@ _intrServiceStallExactList
         vgpuService(pGpu);
     }
 
-    if (bRequiresPossibleErrorNotifier)
-    {
-        //
-        // Notify CUDA there may be an error in ERR_CONT that they may miss because we're
-        // about to clear it out of the NV_CTRL tree backing ERR_CONT before the interrupt
-        // is serviced.
-        //
-        // info32 contains shadowed value of ERR_CONT
-        //
-        gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_POSSIBLE_ERROR, NULL, 0, intrReadErrCont_HAL(pGpu, pIntr), 0);
-    }
-
     if (bIntrStuck)
     {
         NV_PRINTF(LEVEL_ERROR,
@@ -1580,7 +1602,7 @@ intrServiceStallList_IMPL
         }
     }
 
-    resservSwapTlsCallContext(&pOldContext, NULL);
+    NV_ASSERT_OK_OR_ELSE(status, resservSwapTlsCallContext(&pOldContext, NULL), return);
 
     // prevent the isr from coming in
     _intrEnterCriticalSection(pGpu, pIntr, &intrMaskCtx);
@@ -1609,7 +1631,7 @@ done:
     // allow the isr to come in.
     _intrExitCriticalSection(pGpu, pIntr, &intrMaskCtx);
 
-    resservRestoreTlsCallContext(pOldContext);
+    NV_ASSERT_OK(resservRestoreTlsCallContext(pOldContext));
 }
 
 
