@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2016-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2016-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -713,7 +713,7 @@ _scrubCheckAndSubmit
         goto exit;
     }
 
-    _scrubWaitAndSave(pScrubber, pList, pagesToScrubCheck);
+    _scrubWaitAndSave(pScrubber, pScrubListCopy, pagesToScrubCheck);
 
     for (iter = 0; iter < pageCount; iter++)
     {
