@@ -651,12 +651,14 @@ NV_STATUS pmaNumaAllocate
                     status = NV_ERR_NO_MEMORY;
                     break;
                 }
-                pPma->pMapInfo->pmaMapChangeStateAttribEx(pMap, frameOffset, allocOption, MAP_MASK);
+                pPma->pMapInfo->pmaMapChangeStateAttrib(pMap, frameOffset, allocOption, MAP_MASK);
             }
             if (status != NV_OK)
                 break;
         }
 
+        pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
+
         if (status == NV_OK)
         {
             allocationOptions->numPagesAllocated = (NvLength)finalAllocatedCount;
@@ -743,16 +745,18 @@ void pmaNumaFreeInternal
                 //
                 if (currentStatus & STATE_UNPIN)
                 {
-                    pPma->pMapInfo->pmaMapChangeStateAttribEx(pPma->pRegions[regId], (frameNum + j),
-                                                              ATTRIB_NUMA_REUSE, ATTRIB_NUMA_REUSE);
+                    pPma->pMapInfo->pmaMapChangeStateAttrib(pPma->pRegions[regId], (frameNum + j),
+                                                            ATTRIB_NUMA_REUSE, ATTRIB_NUMA_REUSE);
                 }
                 continue;
             }
             sysPagePhysAddr = sysPhysAddr + (j << PMA_PAGE_SHIFT);
             osAllocReleasePage(sysPagePhysAddr, 1 << (PMA_PAGE_SHIFT - osPageShift));
-            pPma->pMapInfo->pmaMapChangeStateAttribEx(pPma->pRegions[regId], (frameNum + j), newStatus, ~ATTRIB_EVICTING);
+            pPma->pMapInfo->pmaMapChangeStateAttrib(pPma->pRegions[regId], (frameNum + j), newStatus, ~ATTRIB_EVICTING);
         }
     }
+
+    pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
 }
 
 void pmaNumaSetReclaimSkipThreshold(PMA *pPma, NvU32 skipReclaimPercent)
